#######################
#    ACPI Monitor     #
# By Adrien Vermeulen #
# Released under GPL  #
#######################

#this import statement allows access to the karamba functions
import karamba
import os
import re

txtBatt = None
txtTemp = None
txtTime = None
txtState = None
fileBatt = "ac.png"
theme = "standart"

def txtColor(widget,colorname):
    global txtBatt, txtTemp, txtTime, txtState
    if colorname == "Black":
        karamba.changeTextColor(widget,txtBatt,0,0,0)
        karamba.changeTextColor(widget,txtTemp,0,0,0)
        karamba.changeTextColor(widget,txtTime,0,0,0)
        karamba.changeTextColor(widget,txtState,0,0,0)
    if colorname == "White":
        karamba.changeTextColor(widget,txtBatt,255,255,255)
        karamba.changeTextColor(widget,txtTemp,255,255,255)
        karamba.changeTextColor(widget,txtTime,255,255,255)
        karamba.changeTextColor(widget,txtState,255,255,255)

#this is called when your widget is initialized
def initWidget(widget):
    global txtBatt
    global txtTime
    global txtTemp
    global txtState
    global imgBatt
    global imgAC
    global imgBorder
    global hBar
    global fileBatt
    global theme, themeColor
    global background
    
    test = karamba.readConfigEntry(widget,"theme")
    if test != None:
        theme = test
    else:
        theme = "standart"
    test = karamba.readConfigEntry(widget,"themeColor")
    if test != None:
        themeColor = test
    else:
        themeColor = "Black"
    
    background = karamba.createBackgroundImage(widget,0,0,"themes/" + theme + "/bg.png")
    
    txtBatt = karamba.createText(widget,101,3,32,11,"")
    karamba.changeTextFont(widget,txtBatt,"verdana")
    karamba.changeTextSize(widget,txtBatt,10)
    
    txtTemp = karamba.createText(widget,133,3,50,11,"")
    karamba.changeTextFont(widget,txtTemp,"verdana")
    karamba.changeTextSize(widget,txtTemp,10)
    
    txtTime = karamba.createText(widget,55,30,150,13,"")
    karamba.changeTextFont(widget,txtTime,"verdana")
    karamba.changeTextSize(widget,txtTime,10)
    
    txtState = karamba.createText(widget,55,3,50,11,"")
    karamba.changeTextFont(widget,txtState,"verdana")
    karamba.changeTextSize(widget,txtState,10)

    txtColor(widget, themeColor)
    
    imgBatt = karamba.createImage(widget,15,0,"gfx/" + fileBatt)
    imgAC = karamba.createImage(widget,25,10,"gfx/plug.png")
    imgBorder = karamba.createImage(widget,66,17,"themes/" + theme + "/hcadre.png")
    hBar = karamba.createImage(widget,67,18,"themes/" + theme + "/hbar.png")
    
    karamba.addMenuConfigOption(widget, "border", "Remove bar border")
    karamba.addMenuConfigOption(widget, "theme", "Select a theme")
    karamba.setMenuConfigOption(widget, "theme", "0")
    
    widgetUpdated(widget)
    

#This is called when your widget is closed.  You can use this to clean
#up open files, etc.  You don't need to delete text and images in your
#theme.  That is done automatically.  This callback is just for cleaning up
#external things.  Most people don't need to put anything here.
def widgetClosed(widget):
    pass

#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
def widgetUpdated(widget):
    global txtBatt
    global txtTime
    global txtTemp
    global txtState
    global imgBatt
    global imgAC
    global hBar
    global fileBatt
    
    f = os.popen("/usr/bin/acpi -t")
    batt = f.readline()
    temp = f.readline()
    f.close()
    f = open("/proc/acpi/ac_adapter/AC/state","r")
    ac = f.readline()
    f.close()
    match = re.search("(dis)?charging, ([0-9]+)%, ([0-9]+):([0-9]+):([0-9]+)?",batt)
    if match == None:
        match = re.search("charged, ([0-9]+)%(, [:0-9]+)?",batt)
        if match.group(1) == "100":
            karamba.changeText(widget, txtBatt, "100%")
            karamba.changeText(widget, txtTime, "charged")
            karamba.resizeImage(widget, hBar, 100,12)
            fileBatt = "ok.png"
            plugState = 1
        else:
            karamba.changeText(widget, txtBatt, "wait...")
    elif match.group(1) == "dis":
        if re.search("on-line", ac):
            karamba.showImage(widget,imgAC)
            karamba.hideImage(widget,imgBatt)
        else:
            plugState = 0
            if int(match.group(2)) < 10:
                fileBatt = "critical.png"
            elif int(match.group(2)) < 20:
                fileBatt = "alert.png"
            elif int(match.group(2)) < 30:
                fileBatt = "warn.png"
            else:
                fileBatt = "ac.png"
        karamba.changeText(widget, txtBatt, match.group(2) + "%")
        karamba.resizeImage(widget, hBar, int(match.group(2)), 12)
        karamba.changeText(widget, txtTime, str(int(match.group(3))) + "h" +  str(int(match.group(4))) + "min remaining")
        karamba.addImageTooltip(widget,imgAC,match.group(0))
    else:
        fileBatt = "ac.png"
        plugState = 1
        karamba.changeText(widget, txtBatt, match.group(2) + "%")
        karamba.resizeImage(widget, hBar, int(match.group(2)), 12)
        if int(match.group(2)) == 100:
            fileBatt = "ok.png"
        karamba.addImageTooltip(widget,imgAC,match.group(0))
        karamba.changeText(widget, txtTime, str(int(match.group(3))) + "h" +  str(int(match.group(4))) + "min until charged")
    karamba.deleteImage(widget, imgBatt)
    karamba.deleteImage(widget, imgAC)
    imgBatt = karamba.createImage(widget,15,0,"gfx/" + fileBatt)
    imgAC = karamba.createImage(widget,25,10,"gfx/plug.png")
    if plugState == 0:
        karamba.hideImage(widget,imgAC)
        karamba.changeText(widget, txtState, "off-line:")
    else:
        karamba.changeText(widget, txtState, "on-line:")
    match = re.search("([0-9.]+) degrees C",temp)
    karamba.changeText(widget, txtTemp,"- " + match.group(1) + "C")
    karamba.redrawWidget(widget)
    

#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
    pass

#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetMouseMoved(widget, x, y, button):
    #Warning:  Don't do anything too intensive here
    #You don't want to run some complex piece of code everytime the mouse moves
    pass


#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):
    pass

#This gets called when an item is clicked in the theme CONFIGURATION menu,
#not the popup menus that you create.
#  key = the reference to the configuration key that was changed
#  value = the new value (true or false) that was selected
def menuOptionChanged(widget, key, value):
    global imgBorder
    global theme
    global themepid
    
    themecommand = ["kdialog", "--title", "Theme Selector", "--radiolist", "Please select a theme to use:", "standart", "Standart", "off", "black", "Dark", "off", "rwhite", "Round White", "off", "rblack", "Round Black", "off"]
    
    if key == "border":
        if value:
            karamba.deleteImage(widget, imgBorder)
        else:
            imgBorder = karamba.createImage(widget,66,17,"themes/" + theme + "/hcadre.png")
    
    elif key == "theme":
        karamba.setMenuConfigOption(widget, "theme", "0")
        themepid = karamba.executeInteractive(widget, themecommand)

    karamba.redrawWidget(widget)

#This gets called when a meter (image, text, etc) is clicked.
# NOTE you must use attachClickArea() to make a meter
# clickable.  
#  widget = reference to your theme
#  meter = the meter clicked
#  button = the button clicked (see widgetClicked for button numbers)
def meterClicked(widget, meter, button):
    pass

#This gets called when a command you have executed with executeInteractive() outputs something
#to stdout.  This way you can get the output of for example kdialog without freezing up the widget
#waiting for kdialog to end.
#  widget = reference to your theme
#  pid = process number of the program outputting (use this if you execute more than out process)
#  output = the text the program outputted to stdout
def commandOutput(widget, pid, output):
    global themepid, hBar, imgBorder, background, theme, themeColor
    
    if pid == themepid:
        theme = output.replace('\n','')
        karamba.writeConfigEntry(widget, "theme", theme)
        karamba.deleteImage(widget, hBar)
        karamba.deleteImage(widget, imgBorder)
        karamba.deleteImage(widget, background)
        background = karamba.createBackgroundImage(widget,0,0,"themes/" + theme + "/bg.png")
        imgBorder = karamba.createImage(widget,66,17,"themes/" + theme + "/hcadre.png")
        hBar = karamba.createImage(widget,67,18,"themes/" + theme + "/hbar.png")
        
        if theme == "black":
            karamba.writeConfigEntry(widget, "themeColor", "White")
            txtColor(widget,"White")
        elif theme == "rwhite":
            karamba.writeConfigEntry(widget, "themeColor", "White")
            txtColor(widget,"White")
        else:
            karamba.writeConfigEntry(widget, "themeColor", "Black")
            txtColor(widget,"Black")
        
        karamba.redrawWidget(widget)

#This gets called when an item is dropped on this widget.
# NOTE you have to call acceptDrops() before your widget will accept drops.
#  widget = reference to your theme
#  dropText = the text of the dropped item (probably a URL to it's location in KDE)
def itemDropped(widget, dropText):
    pass


#This gets called when a new program is LOADING in KDE.  When it is done
#loading, startupRemoved() is called, followed by taskAdded().
#  widget = reference to your widget
#  task = A refence to the task that is starting.  
def startupAdded(widget, startup):
    pass

#This gets called when a new program is done LOADING in KDE.
#  widget = reference to your widget
#  task = A refence to the task that just finished loading.  
def startupRemoved(widget, startup):
    pass

#This is called every time a new task (program) is started in KDE.
#  widget = reference to your widget
#  task = A refence to the new task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def taskAdded(widget, task):
    pass

#This is called everytime a task (program) is closed in KDE.
#  widget = reference to your widget
#  task = A refence to the task.  
def taskRemoved(widget, task):
    pass

#This is called everytime a different task gains focus (IE, the user clicks
#on a different window).  
#  widget = reference to your widget
#  task = A refence to the task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def activeTaskChanged(widget, task):
    pass

# This will be printed when the widget loads.
print "Loaded my python extension!"

