#!/bin/sh
# ColorCustomizer script for the Borealis superkaramba theme ver.1.0
# Copyright 2004 - Ivica Ico Bukvic
# http://meowing.ccm.uc.edu/~ico/
# This install script is made available under GPL license
# For more info regarding this license, please see http://www.gnu.org/copyleft/gpl.html

name='Borealis superkaramba theme ColorCustomizer v.2.0'
title='by Ivica Ico Bukvic
ico@fuse.net
http://meowing.ccm.uc.edu/~ico/
'
directory="/home/$user/Karamba/Borealis"
config="/home/$USER/.kde/share/config"
homepage='http://meowing.ccm.uc.edu/~ico/'
kde_page='http://kde-look.org/content/show.php?content=13876'

#==============================================================

echo -e "\nThe $name $title"

# Check if we are in the proper directory
pwd=`pwd`
if [ "$pwd" != "$HOME/Karamba/Borealis" ]; then
  echo "This script needs to be run from the install folder. Exiting..."
  exit 1
fi

# Check if superkaramba is greater than needed version 0.35
version=`superkaramba -v | grep sup | sed -e 's/.*: //' | sed -e 's/\..*//'`
if [ $version == 0 ]; then
  version=`superkaramba -v | grep sup | sed -e 's/.*\.//'`
  if [ $version -lt 35 ]; then
    echo "This theme requires superkaramba v.0.35 or greater. You are apparently running v.0.${version}. Exiting..."
    exit 1
  fi
fi

if [ $1 ]; then
  doNotCopy=1
else
  doNotCopy=0
fi

echo -e "Configuring background bar's color for the purpose of better legibility. To preview the following choices, please consult the manual. Please select one of the following (1 is default):\n\n1) Highly transparent white bar with a fadeout (good for moderate backgrounds)\n2) Less transparent white bar with a fadeout (good for dark backgrounds)\n3) Highly transparent black bar with a fadeout (good for moderate backgrounds)\n4) Less transparent black bar with a fadeout (good for bright backgrounds)\n5) Highly transparent white bar without a fadeout (good for moderate backgrounds)\n6) Less transparent white bar without a fadeout (good for dark backgrounds)\n7) Highly transparent black bar without a fadeout (good for moderate backgrounds)\n8) Less transparent black bar without a fadeout (good for bright backgrounds)\n\nOr any other key to skip background color customization...\n"

read text
case $text in
  [1] )
    cp -f img/left_light.png img/left.png
    cp -f img/middle_light.png img/middle.png
    cp -f img/monitor_light.png img/monitor.png
  ;;
  [2] )
    cp -f img/left_dark.png img/left.png
    cp -f img/middle_dark.png img/middle.png
    cp -f img/monitor_dark.png img/monitor.png  
  ;;
  [3] )
    cp -f img/left_light_black.png img/left.png
    cp -f img/middle_light_black.png img/middle.png
    cp -f img/monitor_light_black.png img/monitor.png
  ;;
  [4] )
    cp -f img/left_dark_black.png img/left.png
    cp -f img/middle_dark_black.png img/middle.png
    cp -f img/monitor_dark_black.png img/monitor.png
  ;;
  [5] )
    cp -f img/left_light.png img/left.png
    cp -f img/middle_light_full.png img/middle.png
    cp -f img/monitor_light_full.png img/monitor.png
  ;;
  [6] )
    cp -f img/left_dark.png img/left.png
    cp -f img/middle_dark_full.png img/middle.png
    cp -f img/monitor_dark_full.png img/monitor.png
  ;;
  [7] )
    cp -f img/left_light_black.png img/left.png
    cp -f img/middle_light_full_black.png img/middle.png
    cp -f img/monitor_light_full_black.png img/monitor.png
  ;;
  [8] )
    cp -f img/left_dark_black.png img/left.png
    cp -f img/middle_dark_full_black.png img/middle.png
    cp -f img/monitor_dark_full_black.png img/monitor.png
  ;;
  * )
    echo "Skipping background bar customization..."
  ;;
esac
echo -e "...OK\n"


if [ $doNotCopy == 0 ]; then
  echo "Restoring default font color values"
  cp -f misc/cpu.theme.template cpu.theme
  cp -f misc/memory.theme.template memory.theme
  cp -f misc/discs.theme.template discs.theme
  cp -f misc/network.theme.template network.theme
  cp -f misc/network.py.template network.py
  cp -f misc/wireless.theme.template wireless.theme
  cp -f misc/wireless.py.template wireless.py
  cp -f misc/xmms.py.template xmms.py
  echo -e "...OK\n"
fi

echo -e "Configuring font color for the purpose of better legibility. Please select one of the following (1 is default):\n\n1) Black text (0,0,0) (good for lighter backgrounds)\n2) White text (220,220,220) (good for darker backgrounds)\n3) Custom value (R,G,B)\n\nOr any other key to skip color customization...\n"
read text
case $text in
  [1] )
    cat cpu.theme | sed -e 's/color=220,220,220/color=0,0,0/g' > tmp
    mv -f tmp cpu.theme
    cat memory.theme | sed -e 's/color=220,220,220/color=0,0,0/g' > tmp
    mv -f tmp memory.theme
    cat discs.theme | sed -e 's/color=220,220,220/color=0,0,0/g' > tmp
    mv -f tmp discs.theme
    cat network.theme | sed -e 's/color=220,220,220/color=0,0,0/g' > tmp
    mv -f tmp network.theme
    cat network.py | sed -e 's/220, 220, 220/0, 0, 0/g' > tmp
    mv -f tmp network.py
    cat wireless.theme | sed -e 's/color=220,220,220/color=0,0,0/g' > tmp
    mv -f tmp wireless.theme
    cat wireless.py | sed -e 's/220, 220, 220/0, 0, 0/g' > tmp
    mv -f tmp wireless.py
    cat xmms.py | sed -e 's/220, 220, 220/0, 0, 0/g' > tmp
    mv -f tmp xmms.py
  ;;
  [2] )
    cat cpu.theme | sed -e 's/color=0,0,0/color=220,220,220/g' > tmp
    mv -f tmp cpu.theme
    cat memory.theme | sed -e 's/color=0,0,0/color=220,220,220/g' > tmp
    mv -f tmp memory.theme
    cat discs.theme | sed -e 's/color=0,0,0/color=220,220,220/g' > tmp
    mv -f tmp discs.theme
    cat network.theme | sed -e 's/color=0,0,0/color=220,220,220/g' > tmp
    mv -f tmp network.theme
    cat network.py | sed -e 's/0, 0, 0/220, 220, 220/g' > tmp
    mv -f tmp network.py
    cat wireless.theme | sed -e 's/color=0,0,0/color=220,220,220/g' > tmp
    mv -f tmp wireless.theme
    cat wireless.py | sed -e 's/0, 0, 0/220, 220, 220/g' > tmp
    mv -f tmp wireless.py
    cat xmms.py | sed -e 's/0, 0, 0/220, 220, 220/g' > tmp
    mv -f tmp xmms.py
  ;;
  [3] )
    echo "Enter RED value (0-255):"
    read red
    echo "Enter GREEN value (0-255):"
    read green
    echo "Enter BLUE value (0-255):"
    read blue

    sh -c "cat cpu.theme | sed -e 's/color=0,0,0/color=${red},${green},${blue}/g' > tmp"
    mv -f tmp cpu.theme
    sh -c "cat memory.theme | sed -e 's/color=0,0,0/color=${red},${green},${blue}/g' > tmp"
    mv -f tmp memory.theme
    sh -c "cat discs.theme | sed -e 's/color=0,0,0/color=${red},${green},${blue}/g' > tmp"
    mv -f tmp discs.theme
    sh -c "cat network.theme | sed -e 's/color=0,0,0/color=${red},${green},${blue}/g' > tmp"
    mv -f tmp network.theme
    sh -c "cat network.py | sed -e 's/0, 0, 0/${red}, ${green}, ${blue}/g' > tmp"
    mv -f tmp network.py
    sh -c "cat wireless.theme | sed -e 's/color=0,0,0/color=${red},${green},${blue}/g' > tmp"
    mv -f tmp wireless.theme
    sh -c "cat wireless.py | sed -e 's/0, 0, 0/${red}, ${green}, ${blue}/g' > tmp"
    mv -f tmp wireless.py
    sh -c "cat xmms.py | sed -e 's/0, 0, 0/${red}, ${green}, ${blue}/g' > tmp"
    mv -f tmp xmms.py
  ;;  
  * )
    echo "Skipping font customization..."
  ;;
esac
echo -e "...OK\n"

echo -e "Done. Changes you've made will become active upon next (re)load of the theme."

exit 0
