# Script by Ivica Ico Bukvic (SlipStreamScapes)
# http://meowing.ccm.uc.edu/~ico/
# Distributed under the GPL license
# Version 1.0 (July 2004)

import karamba
import os

# width of the desktop, width of the opened bar, bar height, and padding of the retracted menu
desktop = 1280
width = 144
padding = 36
height = 32

#deviceto be monitored
interface = "eth0"

vector = 0
retracted = 0
this = "network"

menuIcons = []
menu = None
menuItems = []

def writeConfigEntryNow(widget, string, replace):
	filename = os.path.expanduser("~") + "/.superkaramba/" + this + ".rc"
	file = open( filename, "r")
	output = file.read()
	file.close()
	output = output.splitlines()
	
	i = 0
	desktop = ""
	for i in output:
		param = i.split()
		if param and param[0].find(string) != -1:
			desktop = desktop + string + "=" + replace + "\n"
		else:
			desktop = desktop + i +"\n"
	
	file = open( filename, "w")
	file.write(desktop)
	file.close

def prettyText(widget, size, font, r, g, b, textname):
    karamba.changeTextSize(widget, textname, size)
    karamba.changeTextFont(widget, textname, font)
    karamba.changeTextColor(widget, textname, r, g, b)

###############################################################################
## Function:    
##   detectResolution ( )
## 
## 
## Description: 
##   Runs xwininfo and sets the current XFree86 resolution to the golbal
##   variables resolutionX and resolutionY
## 
##  Thanks to Adam Geitgey for this code!
###############################################################################
def detectResolution():
	global desktop
	
	havexwininfo = os.system("which xwininfo >/dev/null")
	if (havexwininfo == 0):
		pass
	
	fp = os.popen("xwininfo -root -stats")
	output = fp.read()
	output = output.splitlines()
	
	i = 0
	for i in output:
		param = i.split()
		if (len(param) > 1):
			if param[0].find("Width:") != -1:
				desktop = int(param[1])

##########################################################
# Superkaramba API functions                             #
##########################################################
#this is called when your widget is initialized
def initWidget(widget):
	global desktop
	global padding
	global retracted
	global status
	global lastStatus
	
	global statusImage_on
	global statusImage_off
	global networkImage
	global timer
	
	global mainSwitch
	global menu
	
	global check
	global countdownText

	detectResolution()

	retracted = karamba.readConfigEntry (widget, "retracted")
	x = karamba.readConfigEntry (widget, "widgetPosX")
	y = karamba.readConfigEntry (widget, "widgetPosY")
		
	status = karamba.createText(widget,53, 18, 70, 10, "Disconnected")
	prettyText(widget, 9, "nimbus sans l", 0, 0, 0, status)
	
	countdownText = karamba.createText(widget,24, 18, 10, 10, "")
	prettyText(widget, 8, "nimbus sans l", 255, 255, 255, countdownText)
	
	statusImage_off = karamba.createImage(widget, 23, 19, "img/generic_off.png")
	statusImage_on = karamba.createImage(widget, 23, 19, "img/generic_on.png")
	karamba.hideImage(widget, statusImage_on)
	networkImage = karamba.createImage(widget, 4, 1, "img/network.png")
	
	lastStatus = "Disconnected"
	timer = 0
	mainSwitch = 1
	check = -1
	
	menuIcons.append("img/off.png")
	menuItems.append("")
	
	for i in range (0, 2):
		menuIcons.append("")
		menuItems.append("")

	if ( desktop - x != width ):
		karamba.moveWidget(widget, desktop - width, y)
		writeConfigEntryNow(widget, "widgetPosX", str(x))
		writeConfigEntryNow(widget, "widgetPosY", str(y))
	if ( retracted == 1 ):
		karamba.translateAll(widget,width - padding,0)
		karamba.redrawWidget(widget)
		karamba.toggleWidgetRedraw(widget, 0)

def widgetUpdated(widget):
	global retracted
	global status
	global lastStatus
	global statusImage_off
	global statusImage_on
	global networkImage
	global timer
	global mainSwitch
	global check
	global countdownText
	
	play = 0

	ip = karamba.getIp(widget, interface)
	if ( ip != lastStatus ):
		mainSwitch = 1;
		if ( check > 0 ):
			check = 0
	
	if ( check == 0 ):
		mainSwitch = 1
		check = -1
		karamba.changeText(widget, countdownText, "")
		lastStatus = ""
	
	if ( check > 0 ):
		if ( retracted == 1 ):
			karamba.toggleWidgetRedraw( widget, 1 )
		check = check - 1
		karamba.changeText(widget, countdownText, str(check))
		karamba.redrawWidget( widget )
		if ( retracted == 1 ):
			karamba.toggleWidgetRedraw( widget, 0 )

	if ( timer >= 0 ):
		if ( retracted == 1 ):
			karamba.toggleWidgetRedraw( widget, 1 )
		if ( timer % 2 != 0 ):
			karamba.changeImageIntensity(widget, networkImage, 0.25, 0)
		else:
			karamba.removeImageEffects(widget, networkImage)
		timer = timer - 1
		karamba.redrawWidget(widget)
		if ( retracted == 1 ):
			karamba.toggleWidgetRedraw( widget, 0 )
	
	if ( mainSwitch == 1 ):
		if ( ip == "Disconnected" ) or ( ip == "Error" ):
			mainSwitch = 0
	
		if ( ip != lastStatus ):
			if ( retracted == 1 ):
				karamba.toggleWidgetRedraw( widget, 1 )
			if ( ip != "Disconnected" ) and ( ip != "Error" ):
				karamba.hideImage( widget, statusImage_off )
				karamba.showImage( widget, statusImage_on )
			else:
				karamba.showImage( widget, statusImage_off )
				karamba.hideImage( widget, statusImage_on )
			karamba.changeText( widget, status, ip );
			karamba.redrawWidget(widget)
			if ( retracted == 1 ):
				karamba.toggleWidgetRedraw( widget, 0 )
			lastStatus = ip
			play = 1
		
		if (play == 1):
			os.popen("artsplay $HOME/Karamba/Borealis/sounds/network.wav")
			timer = 5

#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
	global desktop
	global padding
	global height
	global width
	global retracted
	global menu
	global mainSwitch
	global menuItems
	
	if (button == 2):
		menu = karamba.createMenu ( widget )
		if (mainSwitch == 1):
			menuIcons[0] = "img/off.png"
			menuIcons[1] = ""
		else:
			menuIcons[0] = ""
			menuIcons[1] = "img/off.png"
		menuItems[0] = karamba.addMenuItem (widget, menu, "Enable " + interface, karamba.getThemePath(widget) + menuIcons[0])
		menuItems[1] = karamba.addMenuItem (widget, menu, "Disable " + interface, karamba.getThemePath(widget) + menuIcons[1])
		if (mainSwitch == 1):
			menuIcons[2] = "img/clipboard.png"
			menuItems[2] = karamba.addMenuItem (widget, menu, "Copy IP to clipboard", karamba.getThemePath(widget) + menuIcons[2])
		karamba.popupMenu(widget, menu, x, y)
		
	if (button == 1):
		if (retracted == 1) and (x >= width - padding) or (retracted == 0):
			if ( retracted == 1 ):
				retracted = 0			
				karamba.toggleWidgetRedraw( widget, 1 )
				os.popen("artsplay $HOME/Karamba/Borealis/sounds/slide_out_rev2.wav")
				for i in range( width - padding ):
					karamba.translateAll(widget,-1,0)
					karamba.redrawWidget( widget )
			else:
				os.popen("artsplay $HOME/Karamba/Borealis/sounds/slide_in_rev2.wav")
				for i in range( width - padding ):
					karamba.translateAll(widget,1,0)
					karamba.redrawWidget( widget )
				retracted = 1
				karamba.toggleWidgetRedraw( widget, 0 )
			writeConfigEntryNow(widget, "retracted", str(retracted))

def menuItemClicked(widget, menu, id):
	global lastStatus
	global countdownText
	global statusImage_on
	global statusImage_off
	global networkImage
	global timer
	global check
	global mainSwitch
	
	if ( id == menuItems[0] ) and ( mainSwitch != 1):
		tmp = os.popen("/sbin/ifup " + interface + "&");
		if (check >= 0 ):
			karamba.deleteText(widget, countdownText)
		karamba.hideImage(widget, statusImage_on)
		karamba.hideImage(widget, statusImage_off)
		karamba.removeImageEffects(widget, networkImage)
		if (check >= 0 ):
			karamba.changeText(widget, countdownText, "")
		check = 6
		timer = -1
	
	if ( id == menuItems[1] ) and ( mainSwitch != 0):
		os.popen("/sbin/ifdown " + interface);
		if (check >= 0 ):
			karamba.changeText(widget, countdownText, "")
			check = -1
		karamba.removeImageEffects(widget, networkImage)
		timer = -1

	if ( id == menuItems[2] ) and ( mainSwitch == 1 ):
		os.popen("dcop klipper klipper setClipboardContents " + lastStatus)
