# Script by Ivica Ico Bukvic (SlipStreamScapes)
# http://meowing.ccm.uc.edu/~ico/
# Distributed under the GPL license
# Version 1.0 (July 2004)

import karamba
import os

# width of the desktop, width of the opened bar, bar height, and padding of the retracted menu
desktop = 1280
width = 144
padding = 36
height = 32

playing = 0
fx = []
icon = []
current = -1
timer = 0
title = 0
track = 0
lcd = 0
power_led = 0
pause = 0
blinking = 0

menuNames = ( "Toggle Main", "Toggle EQ", "Toggle Playlist", "Toggle All", "Open File(s)", "Properties" )
menuIcons = []
menu = None
menuItems = []

main_window = 0
eq_window = 0
playlist_window = 0

vector = 0
retracted = 0
this = "xmms"

def prettyText(widget, size, font, r, g, b, textname):
    karamba.changeTextSize(widget, textname, size)
    karamba.changeTextFont(widget, textname, font)
    karamba.changeTextColor(widget, textname, r, g, b)

def highlightIcon(widget, num, audio):
	global fx
	global current
	
	if ( current != num ) and (fx[num] == 0):
		karamba.changeImageIntensity(widget, icon[num], 0.25, 0)
		fx[num] = 1
		karamba.redrawWidget(widget)
		#if ( audio == 1 ):
		#	os.popen("artsplay $HOME/Karamba/Borealis/sounds/onButton.wav&")

def restoreIcon(widget, num):
	global fx
	global current
	
	if ( current != num ):
		karamba.removeImageEffects(widget, icon[num])
		fx[num] = 0
		karamba.redrawWidget(widget)

def writeConfigEntryNow(widget, string, replace):
	filename = os.path.expanduser("~") + "/.superkaramba/" + this + ".rc"
	file = open( filename, "r")
	output = file.read()
	file.close()
	output = output.splitlines()
	
	i = 0
	desktop = ""
	for i in output:
		param = i.split()
		if param and param[0].find(string) != -1:
			desktop = desktop + string + "=" + replace + "\n"
		else:
			desktop = desktop + i +"\n"
	
	file = open( filename, "w")
	file.write(desktop)
	file.close
	
def updateText(widget):
	global lcd
	global timer
	global title
	global track
	global current
	global trackNo
	global time
	global playing
	global trackName
	
	if ( playing > 0 ):
		if ( lcd > 0 ):
			lcd = lcd - 1
		else:
			fp = os.popen("xmmsctrl gettime")
			output = fp.read()
			fp.close
			if ( output != time ):
				if (output < 10):
					karamba.changeText(widget, timer, "00:0" + output)
				if (output >= 10) and (output < 60):
					karamba.changeText(widget, timer, "00:" + output)
				if (output >= 60):
					minutes = 0
					while (int(output) > 59):
						if (int(output) > 59):
							output = int(output) - 60
							minutes = minutes + 1
					if (minutes < 10):
						printout = "0" + str(minutes) + ":"
					else:
						printout = str(minutes) + ":"
					if (int(output) < 10):
						karamba.changeText(widget, timer, printout + "0" + str(output))
					else:
						karamba.changeText(widget, timer, printout + str(output))
				time = output
	
		fp = os.popen("xmmsctrl getpos")
		output = fp.read()
		output = output.strip()
		fp.close
		if ( trackNo != output ):
			if ( int(output) < 10 ):
				karamba.changeText(widget, track, "0" + output + ":")
			else:
				karamba.changeText(widget, track, output + ":")
			trackNo = output

		fp = os.popen("xmmsctrl title")
		output = fp.read()
		fp.close
		if ( trackName != output ):
			karamba.changeText(widget, title, output)
			trackName = output
	
	elif ( current != 3 ):
		karamba.changeText(widget, timer, "00:00")
		karamba.changeText(widget, track, "00:")
		karamba.changeText(widget, title, "Xmms is OFF")
		trackNo = 0
		trackName = "Xmms is OFF"
		time = 0

###############################################################################
## Function:    
##   detectResolution ( )
## 
## 
## Description: 
##   Runs xwininfo and sets the current XFree86 resolution to the golbal
##   variables resolutionX and resolutionY
## 
##  Thanks to Adam Geitgey for this code!
###############################################################################
def detectResolution():
	global desktop
	
	havexwininfo = os.system("which xwininfo >/dev/null")
	if (havexwininfo == 0):
		pass
	
	fp = os.popen("xwininfo -root -stats")
	output = fp.read()
	output = output.splitlines()
	
	i = 0
	for i in output:
		param = i.split()
		if (len(param) > 1):
			if param[0].find("Width:") != -1:
				desktop = int(param[1])

##########################################################
# Superkaramba API functions                             #
##########################################################
#this is called when your widget is initialized
def initWidget(widget):
	global desktop
	global padding
	global retracted
	global timer
	global icon
	global title
	global track
	global current
	global power_led
	global pause
	global playing
	global menu
	global menuIcons
	global menuNames
	global trackNo
	global time
	global trackName

	detectResolution()

	retracted = karamba.readConfigEntry (widget, "retracted")
	x = karamba.readConfigEntry (widget, "widgetPosX")
	y = karamba.readConfigEntry (widget, "widgetPosY")
		
	for i in range( 0, 6 ):
		fx.append( 0 )
		
	for i in range (0, len(menuNames)):
		menuIcons.append("img/properties.png")
		menuItems.append("")
		
	menuIcons[3] = ""
	menuIcons[4] = "img/open.png"
	
	trackNo = 0
	time = 0
	trackName = "Xmms is OFF"
	
	icon.append(karamba.createImage(widget, 37, 5, "img/play.png"))
	icon.append(karamba.createImage(widget, 59, 5, "img/rew.png"))
	icon.append(karamba.createImage(widget, 81, 5, "img/ffw.png"))
	icon.append(karamba.createImage(widget, 103, 5, "img/stop.png"))
	icon.append(karamba.createImage(widget, 125, 5, "img/volume_down.png"))
	icon.append(karamba.createImage(widget, 125, 5, "img/volume_up.png"))
	
	power_led = karamba.createImage(widget, 5, 14, "img/xmms_on.png")
	karamba.hideImage(widget, power_led)
	pause = karamba.createImage(widget, 37, 5, "img/pause.png")
	karamba.changeImageIntensity(widget, pause, 0.25, 0)
	karamba.hideImage(widget, pause)
	
	timer = karamba.createText(widget,37, 19, 28, 10, "00:00")
	prettyText(widget, 9, "nimbus sans l", 0, 0, 0, timer)
	track = karamba.createText(widget,65, 19, 16, 10, "00:")
	prettyText(widget, 9, "nimbus sans l", 0, 0, 0, track)
	title = karamba.createText(widget,80, 19, 60, 12, "Xmms is OFF")
	prettyText(widget, 9, "nimbus sans l", 0, 0, 0, title)

	fp = os.popen("xmmsctrl getvol")
	output = fp.read()
	fp.close()

	if ( output == "" ):
		highlightIcon(widget, 3, 0)
		current = 3
	else:
		os.popen("xmmsctrl main 0; xmmsctrl equalizer 0; xmmsctrl play_list 0; xmmsctrl stop")
		current = -1
		playing = 1
		updateText(widget)

	if ( desktop - x != width ):
		karamba.moveWidget(widget, desktop - width, y)
		writeConfigEntryNow(widget, "widgetPosX", str(x))
		writeConfigEntryNow(widget, "widgetPosY", str(y))
	if ( retracted == 1 ):
		karamba.translateAll(widget,width - padding,0)
		karamba.redrawWidget(widget)
		karamba.toggleWidgetRedraw(widget, 0)

def widgetUpdated(widget):
	global retracted
	global playing
	global blinking
	global power_led
	global current

	if ( retracted != 1 ) and ( playing >= 1 ):
		updateText(widget)

	if ( retracted == 1 ) and ( playing >= 2 ) or ( retracted == 0 ) and ( playing == 3 ):
		if ( blinking < 2 ):
			karamba.showImage( widget, power_led )
			blinking = 2
		else:
			karamba.hideImage( widget, power_led )
			blinking = 1
	
	if ( playing > 0 ):
		fp = os.popen("sh -c \"if xmmsctrl playing; then echo 1; else echo 0; fi\"")
		output = fp.read()
		output = output.strip()
		fp.close()

		if ( int(output) == 0 ) and ( playing == 2 ):
			playing = 1
			current = -1
			restoreIcon(widget, 0)
			karamba.hideImage( widget, power_led )
		else:
			if ( int(output) == 1 ) and ( playing == 1 ):
				playing = 2
				highlightIcon(widget, 0, 0)
				karamba.showImage( widget, power_led)
				current = 0

def widgetMouseMoved(widget, x, y, button):
	global fx
	global playing
	
	if ( retracted != 1 ):			
		if (x >= 37) and (x < 56) and (y >= 5) and (y < 20):
			highlightIcon(widget, 0, 1)
		elif ( fx[0] == 1 ):
			restoreIcon(widget, 0)
		
		if (x >= 59) and (x < 78) and (y >= 5) and (y < 20) and (playing > 0):
			highlightIcon(widget, 1, 1)
		elif ( fx[1] == 1 ):
			restoreIcon(widget, 1)
	
		if (x >= 81) and (x < 100) and (y >= 5) and (y < 20) and (playing > 0):
			highlightIcon(widget, 2, 1)
		elif ( fx[2] == 1 ):
			restoreIcon(widget, 2)
	
		if (x >= 103) and (x < 122) and (y >= 5) and (y < 20):
			highlightIcon(widget, 3, 1)
		elif ( fx[3] == 1 ):
			restoreIcon(widget, 3)
	
		if (x >= 125) and (x < 133) and (y >= 10) and (y < 20) and (playing == 2):
			highlightIcon(widget, 4, 1)
		elif ( fx[4] == 1 ):
			restoreIcon(widget, 4)
	
		if (x >= 133) and (x < 140) and (y >= 5) and (y < 11) and (playing == 2):
			highlightIcon(widget, 5, 1)
		elif ( fx[5] == 1 ):
			restoreIcon(widget, 5)

#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
	global current
	global padding
	global height
	global width
	global retracted
	global timer
	global playing
	global lcd
	global power_led
	global pause

	global menu
	global menuNames
	global menuIcons
	global trackNo
	
	if (button == 1):
		if (x >= 37) and (x < 56) and (y >= 5) and (y < 20) and (retracted == 0):
			os.popen("artsplay $HOME/Karamba/Borealis/sounds/click3.wav")
			if ( playing < 1 ):
				os.popen("xmmsctrl launch; xmmsctrl main 0; xmmsctrl equalizer 0; xmmsctrl play_list 0&")
				playing = 1
				restoreIcon(widget, 3)
				current = -1
			elif ( playing == 1) or ( playing == 3):
				os.popen("xmmsctrl play")
				current = 0
				playing = 2
				lcd = 0
				karamba.showImage(widget, power_led)
				karamba.hideImage(widget, pause)
			else:
				os.popen("xmmsctrl pause")
				karamba.showImage(widget, pause)
				playing = 3
		
		elif (x >= 59) and (x < 78) and (y >= 5) and (y < 20) and (playing > 0) and (retracted == 0):
			os.popen("artsplay $HOME/Karamba/Borealis/sounds/click3.wav")
			os.popen("xmmsctrl prev")
			fp = os.popen("xmmsctrl getpos")
			output = fp.read()
			fp.close
			output = output.strip()
			if (output != trackNo):
				updateText(widget)
				if (playing == 3):
					karamba.hideImage(widget, pause)
					playing == 2
					lcd = 0
					current = 0
	
		elif (x >= 81) and (x < 100) and (y >= 5) and (y < 20) and (playing > 0) and (retracted == 0):
			os.popen("artsplay $HOME/Karamba/Borealis/sounds/click3.wav")
			os.popen("xmmsctrl next")
			fp = os.popen("xmmsctrl getpos")
			output = fp.read()
			fp.close
			output = output.strip()
			if (output != trackNo):
				updateText(widget)
				if (playing == 3):
					karamba.hideImage(widget, pause)
					playing == 2
					lcd = 0
					current = 0
	
		elif (x >= 103) and (x < 122) and (y >= 5) and (y < 20) and (retracted == 0):
			if (current == 0):
				os.popen("artsplay $HOME/Karamba/Borealis/sounds/click3.wav")
				os.popen("xmmsctrl stop")
				current = -1
				playing = 1
				restoreIcon(widget, 0)
				lcd = 0
				karamba.hideImage(widget, power_led)
				karamba.hideImage(widget, pause)
			elif (current == -1):
				os.popen("artsplay $HOME/Karamba/Borealis/sounds/click3.wav")
				os.popen("xmmsctrl quit")
				playing = 0
				updateText(widget)
				current = 3
				lcd = 0
	
		elif (x >= 125) and (x < 133) and (y >= 10) and (y < 20) and (playing == 2) and (retracted == 0):
			os.popen("artsplay $HOME/Karamba/Borealis/sounds/click3.wav")
			os.popen("xmmsctrl vol -5")
			fp = os.popen("xmmsctrl getvol")
			output = fp.read()
			output = output.strip()
			fp.close()
			karamba.changeText(widget, timer, output + "%")
			lcd = 2
	
		elif (x >= 133) and (x < 140) and (y >= 5) and (y < 11) and (playing == 2) and (retracted == 0):
			os.popen("artsplay $HOME/Karamba/Borealis/sounds/click3.wav")
			os.popen("xmmsctrl vol +5")
			fp = os.popen("xmmsctrl getvol")
			output = fp.read()
			output = output.strip()
			fp.close()
			karamba.changeText(widget, timer, output + "%")
			lcd = 2

		else:
			if (retracted == 1) and (x >= width - padding) or (retracted == 0):
				if ( retracted == 1 ):
					retracted = 0				
					karamba.toggleWidgetRedraw( widget, 1 )
					os.popen("artsplay $HOME/Karamba/Borealis/sounds/slide_out_rev2.wav")
					for i in range( width - padding ):
						karamba.translateAll(widget,-1,0)
						karamba.redrawWidget( widget )
				else:
					os.popen("artsplay $HOME/Karamba/Borealis/sounds/slide_in_rev2.wav")
					for i in range( width - padding ):
						karamba.translateAll(widget,1,0)
						karamba.redrawWidget( widget )
					retracted = 1
					karamba.toggleWidgetRedraw( widget, 0 )
				writeConfigEntryNow(widget, "retracted", str(retracted))

	if (button == 2):
		if ( playing > 0 ):
			menu = karamba.createMenu ( widget )
			
			if ( main_window == 1 ):
				menuIcons[0] = "img/on.png"
			else:
				menuIcons[0] = "img/off.png"
		
			if ( eq_window == 1 ):
				menuIcons[1] = "img/on.png"
			else:
				menuIcons[1] = "img/off.png"
		
			if ( playlist_window == 1 ):
				menuIcons[2] = "img/on.png"
			else:
				menuIcons[2] = "img/off.png"
		
			for i in range(0, len(menuNames)):
				menuItems[i] = karamba.addMenuItem (widget, menu, menuNames[i], karamba.getThemePath(widget) + menuIcons[i])
	
			karamba.popupMenu(widget, menu, x, y)

#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):
	global playing
	global main_window
	global eq_window
	global playlist_window
		
	if (id == menuItems[0]):
		if ( main_window == 0 ):
			main_window = 1
		else:
			main_window = 0
		os.popen("xmmsctrl main " + str(main_window))
		
	if (id == menuItems[1]):
		if ( eq_window == 0 ):
			eq_window = 1
		else:
			eq_window = 0
		os.popen("xmmsctrl equalizer " + str(eq_window))

	if (id == menuItems[2]):
		if ( playlist_window == 0 ):
			playlist_window = 1
		else:
			playlist_window = 0
		os.popen("xmmsctrl play_list " + str(playlist_window))

	if (id == menuItems[3]):
		if ( main_window == 0 ):
			main_window = 1
			eq_window = 1
			playlist_window = 1
		else:
			main_window = 0
			eq_window = 0
			playlist_window = 0
		os.popen("xmmsctrl main " + str(main_window) + "; xmmsctrl equalizer " + str(eq_window) + "; xmmsctrl play_list " + str(playlist_window))

	if (id == menuItems[4]):
		os.popen("xmmsctrl eject")

	if (id == menuItems[5]):
		os.popen("xmmsctrl pref")

	karamba.deleteMenu(widget, menu)
