# ===========================================================================================
# Needle program for the meter with extra head info image
# Written by Jonny Disenfeldt
# Version 1.0
## ===========================================================================================
import karamba
import math
import os
import string
# -----------------------------------
# Configuration parameters
t1v       = "CPU"                       	# Meter label text
t2v       = "%"                         	# Meter label measure
redlevel  =  80                         	# Sets level for red text
bar_p100_1= 100                                 # Select value for bar value that corresponds to 100%, default 100
scale_t   = [ "0","20","40","60","80","100"]   	# Meter scale
scale_x   = [ 20, 33, 52, 66, 82, 95]   	# X positions
scale_y   = [ 58, 50, 46, 46, 50, 58]   	# Y positions
scale_w   = 30                          	# Width setting
scale_h   = 12                          	# Height setting
# -----------------------------------
needle1img = "image/needle1.png"
needle1    =   0  
degree1    =   0
width1     =   0
height1    =   0

needle2img = "image/needle2.png"
needle2    =   0  
degree2    =   0
width2     =   0
height2    =   0

t1        =   0
t1x       =  55
t1y       =  75
t1w       =  50
t1h       =  15

t2        =   0
t2x       = 100
t2y       = 115
t2w       =  50
t2h       =  15

scale     = [0,0,0,0,0,0]
peak      = -1

bar       = 0

menu1     = 0
id1       = 0 
# -----------------------------------
startdegree = -38
stopdegree  =  38
xoff1       =  61
yoff1       =  61
xoff2       =  61
yoff2       =  61
degreespan  =  stopdegree - startdegree
# ===========================================================================================
# This is called when widget is initialized
def initWidget(widget):
	global needle1
	global width1
	global height1
	global needle2
	global width2
	global height2
	global t1
	global t2
	global scale
	global scale_t
	global scale_x
	global scale_y
	global scale_w
	global scale_h
	global bar
	global menu1
	global id1
	
	menu1   = karamba.createMenu(widget)
	id1     = karamba.addMenuItem(widget, menu1, "Reset Peak Meter", "image/downarrow.png")
	
	bar     = karamba.getThemeBar(widget, "bar_cpu") 
	
	needle1 = karamba.createImage(widget,xoff1,yoff1,needle1img)
	width1  = karamba.getImageWidth(widget,needle1)
	height1 = karamba.getImageHeight(widget,needle1)

	needle2 = karamba.createImage(widget,xoff2,yoff2,needle2img)
	width2  = karamba.getImageWidth(widget,needle2)
	height2 = karamba.getImageHeight(widget,needle2)
	
	t1      = karamba.createText(widget,t1x,t1y,t1w,t1h,t1v)
	t2      = karamba.createText(widget,t2x,t2y,t2w,t2h,t2v)

	for i in range(6):
		scale[i] = karamba.createText(widget,scale_x[i],scale_y[i],scale_w,scale_h,scale_t[i])  
		if int(scale_t[i]) >= redlevel:
	  		karamba.changeTextColor(widget,scale[i],255,0,0)

	karamba.showtext(widget,t1)
	karamba.showtext(widget,t2)
	for i in range(6):
		karamba.showtext(widget,scale[i]);
	
## ===========================================================================================
# This is called when the defined update interval (as specified in the .theme file) has passed
def widgetUpdated(widget):
	global needle1
	global width1
	global height1
	global degree1
	global needle2
	global width2
	global height2
	global degree2
	global peak
	global bar
	
	# CPU usage , pickup the barvalue and adjust for our scale
	percent = int((karamba.getBarValue(widget, bar) *100) / bar_p100_1)
	if percent > 100:
	  percent = 100
	
	degree1 = int( ((degreespan * percent) / 100) - (degreespan / 2) )
	karamba.rotateImage(widget, needle1, int(degree1))
	
	# Get the new w/h , different due to rotation
	w = karamba.getImageWidth(widget,needle1)
    	h = karamba.getImageHeight(widget,needle1)
    	karamba.moveImage(widget, needle1, xoff1 -((w-width1)/2), yoff1 -((h-height1)/2))
	
	if percent > peak:
		peak = percent
		degree2 = int( ((degreespan * peak) / 100) - (degreespan / 2) )
		karamba.rotateImage(widget, needle2, int(degree2))
		w = karamba.getImageWidth(widget,needle2)
  		h = karamba.getImageHeight(widget,needle2)
    		karamba.moveImage(widget, needle2, xoff2 -((w-width2)/2), yoff2 -((h-height2)/2))
	
	karamba.redrawWidget(widget)
# ===========================================================================================
#This gets called everytime our widget is clicked.
#  Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
	global menu1
	
	if (button == 1):
	  karamba.popupMenu(widget, menu1, -100, -100)
# ===========================================================================================
def menuItemClicked(widget, menu, id):
	global menu1
	global id1
	global peak
	global degree2
	
	if (menu == menu1):
		if (id == id1):
			peak = 0
			degree2 = int( ((degreespan * peak) / 100) - (degreespan / 2) )
			karamba.rotateImage(widget, needle2, int(degree2))
			w = karamba.getImageWidth(widget,needle2)
  			h = karamba.getImageHeight(widget,needle2)
    			karamba.moveImage(widget, needle2, xoff2 -((w-width2)/2), yoff2 -((h-height2)/2))
			karamba.redrawWidget(widget)
 
# ===========================================================================================
# Currently not used call-backs
def widgetMouseMoved(widget, x, y, button):
    pass
print "Loaded dmcpu python extension!" 