# ===========================================================================================
# Needle program for the dual meter I/O with extra head info image
# Written by Jonny Disenfeldt
# Version 1.0
## ===========================================================================================
import karamba
import math
import os
import string
# -----------------------------------
# Configuration parameters
t1v       = "eth0"                      	# Meter label text
t2v       = "x100kB/s"                         	# Meter label measure
t3v       = "In"				
t4v       = "Out"
redlevel  =   8                         	# Sets level for red text
bar_p100_1= 1000                                # Select value for bar value that corresponds to 100%, default 1000 (kB/s)
bar_p100_2= 1000                                # Select value for bar value that corresponds to 100%, default 1000 (kB/s)
scale_t1  = [ "0","2","4","6","8","10"]   	# Meter scale
scale_x1  = [ 25, 38, 57, 71, 87,100]   	# X positions
scale_y1  = [ 58, 50, 46, 46, 50, 58]   	# Y positions
scale_w1  = 30                          	# Width setting
scale_h1  = 12                          	# Height setting

scale_t2  = [ "0","2","4","6","8","10"]   	# Meter scale
scale_x2  = [128,141,160,174,190,203]   	# X positions
scale_y2  = [ 58, 50, 46, 46, 50, 58]   	# Y positions
scale_w2  = 30                          	# Width setting
scale_h2  = 12                          	# Height setting
# -----------------------------------
needle1img = "image/needle1.png"
needle1    =   0  
degree1    =   0
width1     =   0
height1    =   0

needle2img = "image/needle2.png"
needle2    =   0  
degree2    =   0
width2     =   0
height2    =   0

needle3img = "image/needle1.png"
needle3    =   0  
degree3    =   0
width3     =   0
height3    =   0

needle4img = "image/needle2.png"
needle4    =   0  
degree4    =   0
width4     =   0
height4    =   0

t1        =   0
t1x       = 110
t1y       = 100
t1w       =  50
t1h       =  15

t2        =   0
t2x       = 100
t2y       = 115
t2w       =  50
t2h       =  15

t3        =   0
t3x       =  60
t3y       =  75
t3w       =  50
t3h       =  15

t4        =   0
t4x       = 160
t4y       =  75
t4w       =  50
t4h       =  15

scale1    = [0,0,0,0,0,0]
scale2    = [0,0,0,0,0,0]
peak1     = -1
peak2     = -1

bars      = [0,0]             # Used for access to the bars defined in the theme !!!

menu1     = 0
id1       = 0 
id2       = 0
id3       = 0
# -----------------------------------
startdegree = -38
stopdegree  =  38
xoff1       =  64
yoff1       =  61
xoff2       =  64
yoff2       =  61
xoff3       = 168
yoff3       =  61
xoff4       = 168
yoff4       =  61
degreespan  =  stopdegree - startdegree
# ===========================================================================================
# This is called when widget is initialized
def initWidget(widget):
	global needle1
	global width1
	global height1
	global needle2
	global width2
	global height2
	global needle3
	global width3
	global height3
	global needle4
	global width4
	global height4
	global t1
	global t2
	global t3
	global t4
	global scale1
	global scale2
	global scale_t1
	global scale_x1
	global scale_y1
	global scale_w1
	global scale_h1
	global scale_t2
	global scale_x2
	global scale_y2
	global scale_w2
	global scale_h2
	global bars
	global menu1
	global id1
	global id2
	global id3
			
	menu1   = karamba.createMenu(widget)
	id1     = karamba.addMenuItem(widget, menu1, "Reset Peak Meter In", "image/downarrow.png")
	id2     = karamba.addMenuItem(widget, menu1, "Reset Peak Meter Out", "image/downarrow.png")
	id3     = karamba.addMenuItem(widget, menu1, "Reset Both Peak Meters", "image/downarrow.png")
	
	bars[0] = karamba.getThemeBar(widget, "bar_in") 
	bars[1] = karamba.getThemeBar(widget, "bar_out") 
	
	needle1 = karamba.createImage(widget,xoff1,yoff1,needle1img)
	width1  = karamba.getImageWidth(widget,needle1)
	height1 = karamba.getImageHeight(widget,needle1)

	needle2 = karamba.createImage(widget,xoff2,yoff2,needle2img)
	width2  = karamba.getImageWidth(widget,needle2)
	height2 = karamba.getImageHeight(widget,needle2)
	
	needle3 = karamba.createImage(widget,xoff3,yoff3,needle3img)
	width3  = karamba.getImageWidth(widget,needle3)
	height3 = karamba.getImageHeight(widget,needle3)
	
	needle4 = karamba.createImage(widget,xoff4,yoff4,needle4img)
	width4  = karamba.getImageWidth(widget,needle4)
	height4 = karamba.getImageHeight(widget,needle4)
	
	t1      = karamba.createText(widget,t1x,t1y,t1w,t1h,t1v)
	t2      = karamba.createText(widget,t2x,t2y,t2w,t2h,t2v)
	t3      = karamba.createText(widget,t3x,t3y,t3w,t3h,t3v)
	t4      = karamba.createText(widget,t4x,t4y,t4w,t4h,t4v)

	for i in range(6):
		scale1[i] = karamba.createText(widget,scale_x1[i],scale_y1[i],scale_w1,scale_h1,scale_t1[i])  
		if int(scale_t1[i]) >= redlevel:
	  		karamba.changeTextColor(widget,scale1[i],255,0,0)
		scale2[i] = karamba.createText(widget,scale_x2[i],scale_y2[i],scale_w2,scale_h2,scale_t2[i])  
		if int(scale_t2[i]) >= redlevel:
	  		karamba.changeTextColor(widget,scale2[i],255,0,0)

	karamba.showtext(widget,t1)
	karamba.showtext(widget,t2)
	karamba.showtext(widget,t3)
	karamba.showtext(widget,t4)
	for i in range(6):
		karamba.showtext(widget,scale1[i]);
		karamba.showtext(widget,scale2[i]);

# ===========================================================================================
# This is called when the defined update interval (as specified in the .theme file) has passed
def widgetUpdated(widget):
	global needle1
	global width1
	global height1
	global degree1
	global needle2
	global width2
	global height2
	global degree2
	global peak1
	global needle3
	global width3
	global height3
	global degree3
	global needle4
	global width4
	global height4
	global degree4
	global peak2
	global bars
	
	# In , pickup the barvalue and adjust for our scale
	percent = int((karamba.getBarValue(widget, bars[0]) *100) / bar_p100_1)
	if percent > 100:
	  percent = 100

	degree1 = int( ((degreespan * percent) / 100) - (degreespan / 2) )
	karamba.rotateImage(widget, needle1, int(degree1))	
	# Get the new w/h , different due to rotation
	w = karamba.getImageWidth(widget,needle1)
    	h = karamba.getImageHeight(widget,needle1)
    	karamba.moveImage(widget, needle1, xoff1 -((w-width1)/2), yoff1 -((h-height1)/2))
	
	if percent > peak1:
		peak1 = percent
		degree2 = int( ((degreespan * peak1) / 100) - (degreespan / 2) )
		karamba.rotateImage(widget, needle2, int(degree2))
		w = karamba.getImageWidth(widget,needle2)
  		h = karamba.getImageHeight(widget,needle2)
    		karamba.moveImage(widget, needle2, xoff2 -((w-width2)/2), yoff2 -((h-height2)/2))
	
	# Out , pickup the barvalue and adjust for our scale
	percent = int((karamba.getBarValue(widget, bars[1]) *100) / bar_p100_2)
	if percent > 100:
	  percent = 100
	
	degree3 = int( ((degreespan * percent) / 100) - (degreespan / 2) )
	karamba.rotateImage(widget, needle3, int(degree3))	
	# Get the new w/h , different due to rotation
	w = karamba.getImageWidth(widget,needle3)
    	h = karamba.getImageHeight(widget,needle3)
    	karamba.moveImage(widget, needle3, xoff3 -((w-width3)/2), yoff3 -((h-height3)/2))
	
	if percent > peak2:
		peak2 = percent
		degree4 = int( ((degreespan * peak2) / 100) - (degreespan / 2) )
		karamba.rotateImage(widget, needle4, int(degree4))
		w = karamba.getImageWidth(widget,needle4)
  		h = karamba.getImageHeight(widget,needle4)
    		karamba.moveImage(widget, needle4, xoff4 -((w-width4)/2), yoff4 -((h-height4)/2))
		
	
	karamba.redrawWidget(widget)
# ===========================================================================================
#This gets called everytime our widget is clicked.
#  Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
	global menu1
	
	if (button == 1):
	  karamba.popupMenu(widget, menu1, -100, -100)
# ===========================================================================================
def menuItemClicked(widget, menu, id):
	global menu1
	global id1
	global id2
	global id3
	global peak1
	global peak2
	global degree2
	global degree4
	
	if (menu == menu1):
		if (id == id1):
			peak1 = 0
			degree2 = int( ((degreespan * peak1) / 100) - (degreespan / 2) )
			karamba.rotateImage(widget, needle2, int(degree2))
			w = karamba.getImageWidth(widget,needle2)
  			h = karamba.getImageHeight(widget,needle2)
    			karamba.moveImage(widget, needle2, xoff2 -((w-width2)/2), yoff2 -((h-height2)/2))
			karamba.redrawWidget(widget)
		elif (id == id2):
			peak2 = 0
			degree4 = int( ((degreespan * peak2) / 100) - (degreespan / 2) )
			karamba.rotateImage(widget, needle4, int(degree4))
			w = karamba.getImageWidth(widget,needle4)
  			h = karamba.getImageHeight(widget,needle4)
    			karamba.moveImage(widget, needle4, xoff4 -((w-width4)/2), yoff4 -((h-height4)/2))
			karamba.redrawWidget(widget)
		else:
			peak1 = 0
			degree2 = int( ((degreespan * peak1) / 100) - (degreespan / 2) )
			karamba.rotateImage(widget, needle2, int(degree2))
			w = karamba.getImageWidth(widget,needle2)
  			h = karamba.getImageHeight(widget,needle2)
    			karamba.moveImage(widget, needle2, xoff2 -((w-width2)/2), yoff2 -((h-height2)/2))
			karamba.redrawWidget(widget)
			peak2 = 0
			degree4 = int( ((degreespan * peak2) / 100) - (degreespan / 2) )
			karamba.rotateImage(widget, needle4, int(degree4))
			w = karamba.getImageWidth(widget,needle4)
  			h = karamba.getImageHeight(widget,needle4)
    			karamba.moveImage(widget, needle4, xoff4 -((w-width4)/2), yoff4 -((h-height4)/2))
			karamba.redrawWidget(widget)					
# ===========================================================================================
# Currently not used call-backs
def widgetMouseMoved(widget, x, y, button):
    pass
print "Loaded dmcpu python extension!" 