#######################
#    WiFi Monitors    #
# By Adrien Vermeulen #
# Released under GPL  #
#######################

import re

#this import statement allows access to the karamba functions
import karamba

#this is called when your widget is initialized
def initWidget(widget):
    global vBar, bar
    global linkText
    global interfaceText
    global levelText
    global noiseText
    
    bartest = karamba.readConfigEntry(widget, "bar")
    if bartest != None:
        bar = bartest
    else:
        bar = "barv_blue.png"
    
    karamba.createBackgroundImage(widget,0,0,"pics/bg.png")
    karamba.createImage(widget,9,5,"pics/wifi.png")
    
    interfaceText = karamba.createText(widget,32,43,38,14,"")
    karamba.changeTextColor(widget,interfaceText,0,0,0)
    karamba.changeTextFont(widget,interfaceText,"verdana")
    
    karamba.createImage(widget,33,58,"pics/cadre_vbar.png")
    
    vBar = karamba.createImage(widget,34,59,"pics/" + bar)
    karamba.resizeImage(widget,vBar,32,0)
    
    linkText = karamba.createText(widget,31,158,38,14,"")
    karamba.changeTextColor(widget,linkText,0,0,0)
    karamba.changeTextFont(widget,linkText,"verdana")
    
    levelText = karamba.createText(widget,22,173,66,11,"level:")
    karamba.changeTextColor(widget,levelText,0,0,0)
    karamba.changeTextFont(widget,levelText,"verdana")
    karamba.changeTextSize(widget,levelText,10)
    
    noiseText = karamba.createText(widget,22,185,66,11,"noise:")
    karamba.changeTextColor(widget,noiseText,0,0,0)
    karamba.changeTextFont(widget,noiseText,"verdana")
    karamba.changeTextSize(widget,noiseText,10)
    
    karamba.addMenuConfigOption(widget, "Bar", "Change the bar")
    karamba.setMenuConfigOption(widget, "Bar", "0")
    
    karamba.widgetUpdated(widget)
    
#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
def widgetUpdated(widget):
    global vBar
    global linkText
    count = 0
    file = open("/proc/net/wireless", "r")
    lines = file.readlines();
    for line in lines:
        count = count + 1
        if count > 2:
            line = line[:-1]
            tokens = re.split(' +', line)
            interface = re.sub(':$', '', tokens[1])
            link = re.sub('\.$', '', tokens[3])
            level = re.sub('\.$', '', tokens[4])
            noise = re.sub('\.$', '', tokens[5])
    karamba.changeText(widget,interfaceText,interface)
    karamba.changeText(widget,levelText,"level: " + level)
    karamba.changeText(widget,noiseText,"noise: " + noise)
    karamba.resizeImage(widget,vBar,32,int(link))
    karamba.moveImage(widget,vBar,34,159 - int(link))
    karamba.changeText(widget,linkText,link + "%")
    karamba.redrawWidget(widget)
    
#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
    pass

#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetMouseMoved(widget, x, y, button):
    #Warning:  Don't do anything too intensive here
    #You don't want to run some complex piece of code everytime the mouse moves
    pass


#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):
    pass

#This gets called when an item is clicked in the theme CONFIGURATION menu,
#not the popup menus that you create.
#  key = the reference to the configuration key that was changed
#  value = the new value (true or false) that was selected
def menuOptionChanged(widget, key, value):
    global barpid
    barpid = None
    barcommand = ["kdialog", "--title", "Bar Selector", "--radiolist", "Please select a bar to use:", "barv_blue.png", "Blue", "off", "barv_green.png", "Green", "off","barv_red.png", "Red", "off", "barv_yellow.png", "Yellow", "off", "barv_purple.png", "Purple", "off", "barv_white.png", "White", "off", "barv_black.png", "Black", "off", "barv_bb.png", "Gradient - Black/Blue", "off", "barv_rg.png", "Gradient - Red/Green", "off", "barv_by.png", "Gradient - Black/Yellow", "off", "barv_tw.png", "Gradient - Transparent/White", "off", "barv_customized.png", "Customized", "off"]
    if (key == "Bar"):
        karamba.setMenuConfigOption(widget, "Bar", "0")
        barpid = karamba.executeInteractive(widget, barcommand)

#This gets called when a command you have executed with executeInteractive() outputs something
#to stdout.  This way you can get the output of for example kdialog without freezing up the widget
#waiting for kdialog to end.
#  widget = reference to your theme
#  pid = process number of the program outputting (use this if you execute more than out process)
#  output = the text the program outputted to stdout
def commandOutput(widget, pid, output):
    global barpid, vBar, bar
    if pid == barpid:
        bar = output.replace('\n','')
        karamba.writeConfigEntry(widget, "bar", bar)
        karamba.deleteImage(widget, vBar)
        vBar = karamba.createImage(widget,34,59,"pics/" + bar)
        karamba.redrawWidget(widget)
        karamba.widgetUpdated(widget)
    
#This gets called when a meter (image, text, etc) is clicked.
# NOTE you must use attachClickArea() to make a meter
# clickable.  
#  widget = reference to your theme
#  meter = the meter clicked
#  button = the button clicked (see widgetClicked for button numbers)
def meterClicked(widget, meter, button):
    pass

# This will be printed when the widget loads.
print "Loaded my python extension!"