#######################
#    WiFi Monitors    #
# By Adrien Vermeulen #
# Released under GPL  #
#######################

import re

#this import statement allows access to the karamba functions
import karamba

#this is called when your widget is initialized
def initWidget(widget):
    global hBar
    global linkText
    global interfaceText
    global levelText
    global noiseText
    
    karamba.createBackgroundImage(widget,0,0,"pics/bg2.png")
    wifiLogo = karamba.createImage(widget,70,5,"pics/wifi_small.png")
    #karamba.resizeImage(widget,wifiLogo,41,20)
    
    interfaceText = karamba.createText(widget,23,11,38,14,"")
    karamba.changeTextColor(widget,interfaceText,80,80,80)
    karamba.changeTextFont(widget,interfaceText,"verdana")
    
    karamba.createImage(widget,24,37,"pics/hbar_cadre.png")
    
    hBar = karamba.createImage(widget,25,38,"pics/hbar.png")
    karamba.resizeImage(widget,hBar,0,15)
    
    linkText = karamba.createText(widget,82,37,37,14,"")
    karamba.changeTextColor(widget,linkText,110,110,110)
    karamba.changeTextFont(widget,linkText,"verdana")
    
    levelText = karamba.createText(widget,133,7,66,11,"level:")
    karamba.changeTextColor(widget,levelText,80,80,80)
    karamba.changeTextFont(widget,levelText,"verdana")
    karamba.changeTextSize(widget,levelText,10)
    
    noiseText = karamba.createText(widget,133,20,66,11,"noise:")
    karamba.changeTextColor(widget,noiseText,80,80,80)
    karamba.changeTextFont(widget,noiseText,"verdana")
    karamba.changeTextSize(widget,noiseText,10)
    
    karamba.widgetUpdated(widget)
    
#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
def widgetUpdated(widget):
    global hBar
    global linkText
    count = 0
    file = open("/proc/net/wireless", "r")
    lines = file.readlines();
    for line in lines:
        count = count + 1
        if count > 2:
            line = line[:-1]
            tokens = re.split(' +', line)
            interface = re.sub(':$', '', tokens[1])
            link = re.sub('\.$', '', tokens[3])
            level = re.sub('\.$', '', tokens[4])
            noise = re.sub('\.$', '', tokens[5])
    karamba.changeText(widget,interfaceText,interface)
    karamba.changeText(widget,levelText,"level: " + level)
    karamba.changeText(widget,noiseText,"noise: " + noise)
    karamba.resizeImage(widget,hBar,int((float(int(link))/100)*150),15)
    karamba.changeText(widget,linkText,link + "%")
    
    karamba.redrawWidget(widget)
    
#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
    pass

#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetMouseMoved(widget, x, y, button):
    #Warning:  Don't do anything too intensive here
    #You don't want to run some complex piece of code everytime the mouse moves
    pass


#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):
    pass

#This gets called when an item is clicked in the theme CONFIGURATION menu,
#not the popup menus that you create.
#  key = the reference to the configuration key that was changed
#  value = the new value (true or false) that was selected
def menuOptionChanged(widget, key, value):
    pass

#This gets called when a command you have executed with executeInteractive() outputs something
#to stdout.  This way you can get the output of for example kdialog without freezing up the widget
#waiting for kdialog to end.
#  widget = reference to your theme
#  pid = process number of the program outputting (use this if you execute more than out process)
#  output = the text the program outputted to stdout
def commandOutput(widget, pid, output):
    pass
    
#This gets called when a meter (image, text, etc) is clicked.
# NOTE you must use attachClickArea() to make a meter
# clickable.  
#  widget = reference to your theme
#  meter = the meter clicked
#  button = the button clicked (see widgetClicked for button numbers)
def meterClicked(widget, meter, button):
    pass

# This will be printed when the widget loads.
print "Loaded my python extension!" 
