# Binary Clock 0.3
# selan at users dot sf dot net
#
# A SuperKaramba binary clock
# based on binclock, written by Brian Gajdos <brian at linuxee dot sk>, 2001
#
#    Copyright (C) 2003
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA


import karamba
import math
import time

class BincColumn:

    def __init__(self, widget, x, y,n):
	self.x = x
	self.y = y
	self.n = n
	self.leds = []
	for i in range(n):
	    self.leds.append(BincLED(widget,x, y + (3 - i) * 15))

    def setup(self, widget,number):
	if self.n > 3:
	    if number & 8:
		self.leds[3].switch_on(widget)
	    else:
		self.leds[3].switch_off(widget)
	if self.n > 2:
	    if number & 4:
		self.leds[2].switch_on(widget)
	    else:
		self.leds[2].switch_off(widget)
	if self.n > 1:
	    if number & 2:
		self.leds[1].switch_on(widget)
	    else:
		self.leds[1].switch_off(widget)
	if number & 1:
	    self.leds[0].switch_on(widget)
	else:
	    self.leds[0].switch_off(widget)

    def destroy(self,widget):
	for i in range(self.n):
		self.leds[i].destroy(widget)

class BincLED:

    def __init__(self, widget, x, y):
	self.x = x
	self.y = y
	self.status = 0
	self.oldstatus = 0
	self.on = karamba.createImage(widget, self.x, self.y, "on.png")
	self.off = karamba.createImage(widget, self.x, self.y, "off.png")
	karamba.hideImage(widget,self.on)
	karamba.hideImage(widget,self.off)

    def switch_on(self,widget):
	self.status = 1
	karamba.hideImage(widget,self.off)
	karamba.showImage(widget,self.on)

    def switch_off(self,widget):
	self.status = 0
	karamba.hideImage(widget,self.on)
	karamba.showImage(widget,self.off)

    def destroy(self,widget):
	karamba.deleteImage(widget,self.on)
	karamba.deleteImage(widget,self.off)

class BincTitle:

    def __init__(self, widget):
		self.status = 0
		self.txthr=karamba.getThemeText(widget,"hr")
		self.txtmin=karamba.getThemeText(widget,"min")
		self.txtsec=karamba.getThemeText(widget,"sec")

    def switch_on(self,widget):
		self.status = 1
		karamba.showText(widget,self.txthr)
		karamba.showText(widget,self.txtmin)
		karamba.showText(widget,self.txtsec)
		karamba.redrawWidget(widget)

    def switch_off(self,widget):
		self.status = 0
		karamba.hideText(widget,self.txthr)
		karamba.hideText(widget,self.txtmin)
		karamba.hideText(widget,self.txtsec)
		karamba.redrawWidget(widget)

class BincBack:

    def __init__(self, widget):
		self.status = 0
		self.image=karamba.getThemeImage(widget,"frame")

    def switch_on(self,widget):
		self.status = 1
		karamba.showImage(widget,self.image)
		karamba.redrawWidget(widget)

    def switch_off(self,widget):
		self.status = 0
		karamba.hideImage(widget,self.image)
		karamba.redrawWidget(widget)

class BincDigital:

    def __init__(self, widget):
	self.status = 0
	self.text=karamba.getThemeText(widget,"digi")

    def switch_on(self,widget):
	self.status = 1
	karamba.showText(widget,self.text)
	karamba.redrawWidget(widget)

    def switch_off(self,widget):
	self.status = 0
	karamba.hideText(widget,self.text)
	karamba.redrawWidget(widget)

    def update(self,widget,clock):
	if clock==1:
		strclock="%H:%M:%S"
	else:
		strclock="%I:%M:%S"
	karamba.changeText(widget,self.text,time.strftime(strclock))
	karamba.redrawWidget(widget)

class BincClock:
	def __init__(self, widget):
		self.is24=1
		self.leds=1
		self.bincols=[]

	def setup(self, widget):

		if self.leds==1:
			colleds=[4,4,4,4,4,4]
		else:
			colleds=[2,4,3,4,3,4]

		self.bincols = [
			BincColumn(widget,10, 20,colleds[0]),
			BincColumn(widget,25, 20,colleds[1]),
			BincColumn(widget,50, 20,colleds[2]),
			BincColumn(widget,65, 20,colleds[3]),
			BincColumn(widget,90, 20,colleds[4]),
			BincColumn(widget,105, 20,colleds[5])
		]

		karamba.redrawWidget(widget)

	def reset(self, widget):
		for col in self.bincols:
			col.destroy(widget)


#this is called when you widget is initialized
def initWidget(widget):
	global bintitle,bindigi,binback,binc

	binc=BincClock(widget)
	binback=BincBack(widget)
	bintitle=BincTitle(widget)
	bindigi=BincDigital(widget)

	karamba.addMenuConfigOption(widget,"bg","Show background")
	karamba.addMenuConfigOption(widget,"titles","Show titles")
	karamba.addMenuConfigOption(widget,"digital","Show digital clock")
	karamba.addMenuConfigOption(widget,"leds","Show extra LEDs")
	karamba.addMenuConfigOption(widget,"24","Use 24-hour clock")

	bg = karamba.readConfigEntry(widget, "bg")
	if bg == None:
		bg=1
	menuOptionChanged(widget,"bg",bg)

	titles = karamba.readConfigEntry(widget, "titles")
	if titles == None:
		titles=1
	menuOptionChanged(widget,"titles",titles)

	is24=karamba.readConfigEntry(widget, "24")
	if is24==None:
		binc.is24=1

	digital=karamba.readConfigEntry(widget, "digital")
	if digital==None:
		digital=1
	menuOptionChanged(widget,"digital",digital)

	leds=karamba.readConfigEntry(widget, "leds")
	if leds==None:
		leds=1
	binc.leds=leds
	binc.setup(widget)

	widgetUpdated(widget)


#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
def widgetUpdated(widget):

	cas2 = time.localtime()
	year, month, day, hour, minute, second, weekday = cas2[:7]

	if binc.is24==0:
		if hour>12:
			hour=hour-12
		if hour==0:
			hour=12
	if bindigi.status==1:
		bindigi.update(widget,binc.is24)
	binc.bincols[0].setup(widget,hour / 10)
	binc.bincols[1].setup(widget,hour % 10)
	binc.bincols[2].setup(widget,minute / 10)
	binc.bincols[3].setup(widget,minute % 10)
	binc.bincols[4].setup(widget,second / 10)
	binc.bincols[5].setup(widget,second % 10)
	karamba.redrawWidget(widget)


#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
    pass

#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetMouseMoved(widget, x, y, button):
    #Warning:  Don't do anything too intensive here
    #You don't want to run some complex piece of code everytime the mouse moves
    pass


#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):
    pass

#This gets called when an item is clicked in the theme CONFIGURATION menu,
#not the popup menus that you create.
#  key = the reference to the configuration key that was changed
#  value = the new value (true or false) that was selected
def menuOptionChanged(widget, key, value):

	#print key
	#print value

	if key=="bg":
		if value==1:
			binback.switch_on(widget)
		else:
			binback.switch_off(widget)
		karamba.writeConfigEntry(widget, "bg", str(binback.status))
	elif key=="titles":
		if value==1:
			bintitle.switch_on(widget)
		else:
			bintitle.switch_off(widget)
		karamba.writeConfigEntry(widget, "titles", str(bintitle.status))
	elif key=="digital":
		if value==1:
			bindigi.switch_on(widget)
			bindigi.update(widget,binc.is24)
		else:
			bindigi.switch_off(widget)
		karamba.writeConfigEntry(widget, "digital", str(bindigi.status))
	elif key=="leds":
		binc.leds=value
		binc.reset(widget)
		binc.setup(widget)
		widgetUpdated(widget)
		karamba.writeConfigEntry(widget, "leds", str(value))
	elif key=="24":
		binc.is24=value
		karamba.writeConfigEntry(widget, "24", str(value))
		widgetUpdated(widget)

#This gets called when a meter (image, text, etc) is clicked.
# NOTE you must use attachClickArea() to make a meter
# clickable.
#  widget = reference to your theme
#  meter = the meter clicked
#  button = the button clicked (see widgetClicked for button numbers)
def meterClicked(widget, meter, button):
    pass

#This gets called when a command you have executed with executeInteractive() outputs something
#to stdout.  This way you can get the output of for example kdialog without freezing up the widget
#waiting for kdialog to end.
#  widget = reference to your theme
#  pid = process number of the program outputting (use this if you execute more than out process)
#  output = the text the program outputted to stdout
def commandOutput(widget, pid, output):
    pass

#This gets called when an item is dropped on this widget.
# NOTE you have to call acceptDrops() before your widget will accept drops.
#  widget = reference to your theme
#  dropText = the text of the dropped item (probably a URL to it's location in KDE)
def itemDropped(widget, dropText):
    pass


#This gets called when a new program is LOADING in KDE.  When it is done
#loading, startupRemoved() is called, followed by taskAdded().
#  widget = reference to your widget
#  task = A refence to the task that is starting.
def startupAdded(widget, startup):
    pass

#This gets called when a new program is done LOADING in KDE.
#  widget = reference to your widget
#  task = A refence to the task that just finished loading.
def startupRemoved(widget, startup):
    pass

#This is called every time a new task (program) is started in KDE.
#  widget = reference to your widget
#  task = A refence to the new task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def taskAdded(widget, task):
    pass

#This is called everytime a task (program) is closed in KDE.
#  widget = reference to your widget
#  task = A refence to the task.
def taskRemoved(widget, task):
    pass

#This is called everytime a different task gains focus (IE, the user clicks
#on a different window).
#  widget = reference to your widget
#  task = A refence to the task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def activeTaskChanged(widget, task):
    pass

# This will be printed when the widget loads.
print """
Binary Clock version 0.3, Copyright (C) 2003 selan at users dot sf dot net
This software comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions. See the included COPYING file for details.
"""
