import re
###############################################################################
## Function:
##   readConfigurationFile ( string $filename )
##
##
## Description:
##   Reads the file $filename, and stores information from the file into
##   the global hash %conf.  This version of the function has been modified
##   specifically to accomadate the needs of Kroller.
##
## Example:
##   readConfigurationFile("/etc/configuration.conf");
##
###############################################################################
def XBarReadCfg(filename,struttura):
	## Open the configuration file
	try:
		file = open(filename,"r")
	except IOError:
		print "ERROR While opening the configuration file [" + filename + "]"
	## Read through each line of the file
	while 1:
		line = file.readline()                        ## Read a line from the file
		if len(line) == 0:                            ## Break this while loop if we're at the end of the file
			break
		line = re.sub("#.*$", "", line)               ## Remove comments
		line = re.sub("//.*$", "", line)              ## Remove comments
		line = line.strip()                           ## Remove line ending characters, and extra whitespace
		if re.search("\s+=>\s+", line):               ## If it looks like a valid configuration line
			## Extract the name/value pair
			(name, value) = re.split("\s+=>\s+", line, 1)
			if (name == "addIconCommand"):
				struttura["cmdIcons"].append(value)
			elif (name == "addTrayIcon"):
				struttura["imgIcons"].append(value)
			elif (name == "addMenuType"):
				struttura["typeMenu"].append(value)
			else:
				## Must be a generic configuration parameter, store it in conf
				if (re.match("^\d+$", value)):
					struttura[name] = int(value)
				else:
					struttura[name] = value
	## Close the coniguration file
	file.close()


###############################################################################
## Function:
##   getKDEConfigValue ( string $filename, string $section, string $option )
##
##
## Description:
##   Reads the file $filename, searches for the section $section (each section
##   is delimeted by  "[section name]" in KDE config files, finds the option
##   $option and returns the value.
##
## Example:
##   num = getKDEConfigValue("/root/.kde/share/kdeglobals", "Desktops", "Number");
##
###############################################################################
def getKDEConfigValue(filename, section, option):
    """Read a KDE config file and return the value for option under the specified section"""

    ## FIXME: Should this function be replaced with calls to
    ## pythons ConfigParser?

    ## Open the configuration file
    try:
        file = open(filename,"r")
    except IOError:
        print
        print "ERROR While opening the configuration file [" + filename + "]"
        print

    ## Read through each line of the file
    while 1:
        line = file.readline()                        ## Read a line from the file
        if len(line) == 0:                            ## Break this while loop if we're at the end of the file
            break

        line = re.sub("#.*$", "", line)               ## Remove comments
        line = re.sub("//.*$", "", line)              ## Remove comments
        line = line.strip()                           ## Remove line ending characters, and extra whitespace
        if re.search("^\[" + section + "\]", line):           ## If we're entering the right section
            while 1:
                line = file.readline()
                if len(line) == 0:                            ## Break this while loop if we're at the end of the file
                    break
                line = line.strip()
                if re.search("^" + option + "=", line):
                    (name, value) = re.split("=", line, 1)
                    file.close
                    return(str(value))

    ## Close the coniguration file and return an error if we didn't find what we were looking for
    file.close()
    return(-1)




