######################################################################
# Programmer: Ryan Nickell
# Filename: rpmCpu.py
# Date: June 06, 2004
# Release version: 0.2
# Notes: This is a cpu monitor application using a tachometer image.
#	It is designed to go from 0 to 8,000rpms in correlation to
#	0-100% cpu usage.
# License: GPL
#
# Requirements:
#	Qt: 3.1.1
#	KDE: 3.1.0
#	Superkaramba: 0.32 (it may work on older versions, untested)
#
######################################################################
#this import statement allows access to the karamba functions
import karamba

#globals
debug = 0
deg = 0
cpu = 0
#system = 0
width=137
height=137
preProg,prevIdle = 0,0
fontSize = 10
font = "terminal"
#font colors that can be used
#white, black, blue, red, green, yellow, purple, tan, gray
fontColor = "white"
shadow = 0
####################################################
#text modification
#More colors will be added later.. right now we have
#support for: white, black, blue, red, green, yellow, purple, tan, gray
#more can be found at:
# http://hokum.freehomepage.com/Content/Reference/Ref_RGB.html
####################################################
def prettyText(widget, size, font, textname, fontColor, shadow):
	karamba.changeTextSize(widget, textname, size)
	karamba.changeTextFont(widget, textname, font)
	if(fontColor == "white"):
		karamba.changeTextColor(widget, textname, 255, 255, 255)
	elif(fontColor == "black"):
		karamba.changeTextColor(widget, textname, 0, 0, 0)
	elif(fontColor == "blue"):
		karamba.changeTextColor(widget, textname, 0, 0, 255)
	elif(fontColor == "red"):
		karamba.changeTextColor(widget, textname, 255, 0, 0)
	elif(fontColor == "green"):
		karamba.changeTextColor(widget, textname, 0, 255, 0)
	elif(fontColor == "yellow"):
		karamba.changeTextColor(widget, textname, 255, 255, 0)
	elif(fontColor == "purple"):
		karamba.changeTextColor(widget, textname, 127, 0, 127)
	elif(fontColor == "tan"):
		karamba.changeTextColor(widget, textname, 127, 127, 0)
	elif(fonColr == "gray"):
		karamba.changeTextColor(widget, textname, 0,127,127)
	else:
		karamba.changeTextColor(widget, textname, 0,0,0)
	karamba.changeTextShadow(widget, textname, shadow)

################################################
# cpuLoad(): generates the cpu load by querying
# /proc/stat and calculating the difference
################################################
def cpuLoad():
	global cpu,preProg,prevIdle,debug
	try:
		#kernel 2.4 uses this format
		(junk,user,nice,system,idle) = file('/proc/stat').readline().split()
	except:
		#kernel 2.6 uses this format
		(junk,user,nice,system,idle,junk2,junk3,junk4) = file('/proc/stat').readline().split()
	prog = long(user)+long(nice)+long(system)
	idle = long(idle)
	idleChange = idle - prevIdle
	progChange = prog - preProg
	load = progChange * 100.0 / (idleChange + progChange)
	preProg, prevIdle = prog, idle
	#now update the global cpu variable to be read by other callbacks
	cpu = load
	if(debug >=1):
		print "cpuLoad(): load:",load,"idleChange:",idleChange,"progChange:",progChange
		print "cpuLoad(): junk:",junk,"user",user,"nice:",nice,"system:",system,"idle:",idle
		print "cpuLoad(): cpu=",progChange,"/",idleChange,"+",progChange,"=>",load

#this is called when you widget is initialized
def initWidget(widget):
	global image,system,cpu
	background = karamba.createBackgroundImage(widget, 0, 0, "tach.png")
	image = karamba.createImage(widget, 0,0,"arm.png")
	text = "%03i" % cpu
	system = karamba.createText(widget, 57, 100, 40, 20, text+"%")
	prettyText(widget, fontSize, font, system, fontColor, shadow)
	karamba.redrawWidget(widget)

#this is called everytime your widget is updated
#the update inverval is specified in the .theme filed
def widgetUpdated(widget):
	global image,deg,cpu,system
	cpuLoad()
	karamba.deleteText(widget, system)
	deg = (cpu/100) * 265
	karamba.rotateImage(widget, image, int(deg))
	#the rotateImage call only works for 0,90,180,270,360 degrees
	#the following code allows this to work with all decimal values
	#integer or double/floating point.
	w = karamba.getImageWidth(widget,image)
	h = karamba.getImageHeight(widget,image)
	karamba.moveImage(widget,image,0-((w-width)/2),0-((h-height)/2))
	text = "%03i" % cpu
	system = karamba.createText(widget, 57, 100, 40, 20, text+"%")
	prettyText(widget, fontSize, font, system, fontColor, shadow)
	karamba.redrawWidget(widget)

