###############################################################
#  Copyright by Charles Wienken
#  Licensed Under GPL
###############################################################

###############################################################
#  This import statement allows access to the karamba functions
###############################################################
import karamba
import commands
import string
import time
import sys
import gc

#xBar modules
import xBar
from xBar import *
from xBar.sysTray import xSysTray
from xBar.currTasks import xCurrTasks

###############################################################
#  Globals
###############################################################
spacer = 10
filledR = 0 # filled right
filledL = 0 # filled Left
bottom = 0 # bottom variable

###############################################################
#  Classes
###############################################################

class imgObj:
    # initialize the object
    def __init__(self,name,cmd,imgPath,width,height,reSize=0,yPos=2,xPos=50,color=1):
	self.__name = name # name of the obj
	self.__cmd = cmd # obj command to execute
	self.__id = 0 # ID of Obj
	self.__imgID = 0 # ID of the background
	self.__imgPath = imgPath # path to image
        self.__hidden = 0 # not hidden by default, but autodetect later
	self.__xPos = xPos # x position
	self.__yPos = yPos # y position
	self.__width = width # width of the obj
	self.__height = height # height of the obj
	self.__resize = reSize # if 1 (true resize), otherwise don't
	self.__color = color # if 1 (true color can change), otherwise don't
	
    # get the width of the object
    def __len__(self):
	length = 0
	if( self.__hidden == 0):
	    length = self.__width
	return length
	    
    # destroy the object
    def destroy(self, widget):
        try:
	    karamba.deleteImage(widget,self.__imgID)
	except:
	    'Could not delete image object'
	    
    # set the width of the object
    def resetWidth(self,newWidth):
	self.__width = newWidth
    # create the obj
    def create(self,widget,xpos):
        self.__xPos=xpos
        self.__id=karamba.createImage(widget,self.__xPos,self.__yPos,self.__imgPath+self.__name+".png" )
	if( self.__resize == 1): 
	    karamba.resizeImageSmooth( widget, self.__id, self.__height, self.__height )
	karamba.attachClickArea(widget, self.__id, self.__cmd, "", "")
    
    # change the color of the image
    def color(self,widget,color):
	if value == 1:
	    karamba.changeImageChannelIntensity(widget, self.__id, 1, 33, 0)
    
    # change the visibility
    def visible(self,widget,option):
	if( option == 0 ):
	    self.__hidden = 1
	    # hide the Image
	    karamba.hideImage( widget, self.__id )
	else:
	    self.__hidden = 0
	    # hide the Image
	    karamba.showImage( widget, self.__id )
    # move the image
    def move(self,widget,xpos):
	self.__xPos = xpos
	karamba.moveImage(widget, self.__id, self.__xPos, self.__yPos)
	    
class configObj:
    # initialize the configure menu
    def __init__(self, newName, newDisplay):
	self.__name = newName # config menu name
	self.__display = newDisplay # config menu display
    
    # create the configure menu option
    def create(self,widget):
        karamba.addMenuConfigOption(widget, self.__name, self.__display)
    
    # return the name of the object
    def getName(self):
	return self.__name 
	
class objDict:
    # initialize the object
    def __init__(self):
        self.__objectDict = {}
    # add an obj with given key
    def __setitem__(self,key,newObj):
        self.__objectDict[key] = newObj
    # get the obj with given key
    def __getitem__(self,key):
	return self.__objectDict[key]
    # get the number of objects in list
    def __len__(self):
	length = 0
	list = self.__objectDict
	for k in list.keys():
	    length = length + len(list[k])
	return length 
    # return the list of keys
    def keys(self):
	return self.__objectDict.keys()
    # sort the list
    def sort(self):
	self.__objectDict.sort()
    def has_key(self,k):
	return self.__objectDict.has_key(k) 

###############################################################
#  xBgBar Class
###############################################################

class xBgBar:  
    # initialize the bar with the following
    def __init__(self, newThemeName, newImgPath, newMinWidth, newHeight, newSideWidth ):
        self.__themeName = newThemeName # name of the theme
	self.__midID = 0 # Id of the middle port
	self.__rightID = 0 # ID of the right port
	self.__leftID = 0 # ID of the left port
	self.__sideWidth = newSideWidth # width of each side
	self.__minWidth = newMinWidth # minimum width of the bar
	self.__maxWidth = 1600 # max width is set to the resolution (should autodetect)
	self.__maxHeight = 1200 # max height is set to the resolution (should autodetect)
	self.__width = newMinWidth # width is at least the min Width
	self.__height = newHeight # height of the bar
	self.__top = 1 # bar is a top bar (changes from the theme file)
	self.__themeName = newThemeName # the name of the Theme
	self.__imgPath = newImgPath # name of path to image
	
    # get the width of the object	    
    def __len__(self):
	return self.__width # return the width of the bars
	
    # destroy the object
    def destroy(widget, self):
        try:
	    karamba.deleteImage( widget,self.__leftID)
	    karamba.deleteImage( widget,self.__rightID)
	    karamba.deleteImage( widget,self.__midID)
	except:
	    'Could not delete bar objects'	
	
    # hide the object
    def hide( self, widget ):
	karamba.hideImage( widget,self.__leftID)
	karamba.hideImage( widget,self.__rightID)
	karamba.hideImage( widget,self.__midID)
    
    # hide the object
    def show( self, widget ):
	karamba.showImage( widget,self.__leftID)
	karamba.showImage( widget,self.__rightID)
	karamba.showImage( widget,self.__midID)	
	
    # get the width of the sides
    def getSideWidth(self):
	return self.__sideWidth # return the width of each side
	    
    # set the widget width and position
    def set(self,widget):
        global bottom 
	
	kThemePath=commands.getoutput("cat "+karamba.getThemePath(widget)+self.__themeName )
        for l in kThemePath.splitlines():
            param = l.split()
            for i in range( len(param) ):
                if param[i].find("W=") <> -1:
                    self.__width = int(param[i].strip("W="))
                elif param[i].find( "BOTTOMBAR" ) <> -1:
                    bottombar = string.upper( param[i].strip("BOTTOMBAR") )
                elif param[i].find( "TOPBAR" ) <> -1:
                    topbar = string.upper( param[i].strip("TOPBAR") )
        # Detect screen width and height 
        xWinInfo = commands.getoutput( "xwininfo -root -stats" )
        for l in xWinInfo.splitlines():
            param = l.split()
            if( len(param) > 1 ) and param[0].find("Width:") <> -1:
                self.__maxWidth = int(param[1])
            if( len(param) > 1 ) and param[0].find("Height:") <> -1:
                self.__maxHeight = int(param[1])
        # do a sanity check on the width
        if( self.__width < self.__minWidth ):
            self.__width = self.__minWidth
        elif( self.__width > self.__maxWidth ):
            self.__width = self.__maxWidth
        # resize the widget   
        karamba.resizeWidget( widget, self.__width, self.__height )
        # move the widget to the bottom
        if bottombar == "=TRUE" and topbar == "=FALSE":
            karamba.moveWidget( widget,
	    (self.__maxWidth-self.__width)/2,self.__maxHeight-self.__height )
	    bottom = 1
        else:
            karamba.moveWidget( widget,(self.__maxWidth-self.__width)/ 2, 0)
	    bottom = 0

    # create the bars
    def create(self,widget):
            #  Left port
            self.__leftID=karamba.createImage(widget,0,0,self.__imgPath+"bar_left.png")
            karamba.resizeImageSmooth(widget,self.__leftID,self.__sideWidth,self.__height)
    
            #  Middle port
            self.__midID=karamba.createImage(widget,self.__sideWidth,0,self.__imgPath+"bar_middle.png")
            karamba.resizeImageSmooth(widget,self.__midID,self.__width-self.__sideWidth*2,self.__height)

            #  Right port
            self.__rightID=karamba.createImage(widget,self.__width-self.__sideWidth,0,self.__imgPath+"bar_right.png" )
            karamba.resizeImageSmooth(widget,self.__rightID,self.__sideWidth,self.__height) 

###############################################################
#  xDateTime Class
###############################################################

class xDateTime:    
    #  initialize the date and time
    def __init__(self,FSize,FFamily,FColor,cmd,imgPath,xpos,ypos,imgXpos,imgYpos,width,height):
        self.__fontSize = FSize # size of the font
	self.__fontFamily = FFamily # font family 
        self.__fontColor = FColor # font color (white is 255, black is 0)
        self.__id = 0 # ID of Obj
	self.__imgID = 0 # ID of the background
	self.__name = "datetime" # name of the obj
        self.__cmd = cmd # obj command to execute
        self.__imgPath = imgPath # path to image
        self.__hidden = 0 # not hidden by default, but autodetect later
        self.__xPos = xpos # x position
        self.__yPos = ypos # y position
        self.__imgXpos = imgXpos # img x position
        self.__imgYpos = imgYpos # img y position
        self.__width = width # width of the obj
        self.__height = height # height of the obj
	
    # get the width of the date
    def __len__(self):
	length = 0
	if( self.__hidden == 0 ):
	    length = self.__width
	return length 

    # destroy the object
    def destroy(self, widget):
        try:
	    karamba.deleteImage( widget,self.__imgID)
	except:
	    'Could not delete datetime image object'
	
    # create the date and time
    def create(self, widget, xpos):
	global spacer
	self.__imgXpos = xpos - self.__width
	self.__xPos = self.__imgXpos + int(spacer)
        self.__imgID=karamba.createImage(widget,self.__imgXpos,self.__imgYpos,self.__imgPath+self.__name+".png")
	localTime = time.strftime('%b %d, %Y   %I:%M %P', time.localtime() )	
        self.__id=karamba.createText(widget,self.__xPos,self.__yPos,self.__width,self.__height,localTime)
        karamba.attachClickArea(widget, self.__id, self.__cmd, "", "")  
        karamba.changeTextShadow(widget, self.__id, 0 ) 
        karamba.changeTextColor(widget,self.__id,self.__fontColor,self.__fontColor,self.__fontColor )
        karamba.changeTextSize(widget, self.__id, self.__fontSize )
	karamba.changeTextFont(widget, self.__id, self.__fontFamily)
	
    # update the local time
    def update(self, widget):
	if( self.__hidden == 0):
	   localTime=time.strftime('%b %d, %Y   %I:%M %P', time.localtime())# Get the time Now()
    	   karamba.changeText( widget, self.__id, localTime )

    # change the color of the text
    def color(self,widget,color):
	c1 = 255
	c2 = 255
	c3 = 255
	    
	if( color == "black" ):
	    c1 = 0
	    c2 = 0
	    c3 = 0
	elif( color== "red" ):
	    c1 = 255
	    c2 = 0
	    c3 = 0
	elif( color == "purple" ):
	    c1 = 117
	    c2 = 0
	    c3 = 146
	elif( color == "blue" ):
	    c1 = 11
	    c2 = 11
	    c3 = 193
	elif( color == "green" ):
	    c1 = 10
	    c2 = 145
	    c3 = 64
	elif( color == "orange" ):
	    c1 = 255
	    c2 = 168
	    c3 = 0
	
	karamba.changeTextColor(widget, self.__id, c1, c2, c3)
	
    # change the visibility of the datetime
    def visible(self,widget,option):
	if( option == 0 ):
	    self.__hidden = 1
	    # change text to ""
	    karamba.changeText( widget, self.__id, "" )
	    # hide the background
	    karamba.hideImage( widget, self.__imgID )
	    # move clock Text off the screein
	    karamba.moveText( widget, self.__id, -250, self.__yPos)
	else:
	    self.__hidden = 0
	    # update the time
	    self.update( widget )
	    # move text to the original position
	    karamba.moveText( widget, self.__id, self.__xPos, self.__yPos)
	    # show the date background
	    karamba.showImage( widget, self.__imgID ) 

###############################################################
#  Logger Class
###############################################################

# used to log all output from xBar to a file for later use.
class xLogger:
	#FileObject representing the log file
	logFile = None

	#CONSTRUCTOR/DESTRUCTOR
	def __init__(self,widget):
		#create FileObject for the log file
		self.logFile = file(karamba.getThemePath(widget) + "xBar.log", "w")

		#output initial log info
		self.logFile.write(" xBar v2.0 Log File\n")
		self.logFile.write("-------------------------\n")
		self.logFile.write(time.asctime() + "\n\n")

	def __del__(self):
		#close FileObject for the log file
		self.logFile.close()

	#METHODS
	#** void write(string output)
	# writes the given output to both stdout and the logfile
	def write(self, output):
		#write to stdout
		sys.__stdout__.write(output)

		#write to logfile
		self.logFile.write(output)
		self.logFile.flush() 
###############################################################
#  Core xBar Class
###############################################################
class xBarcore:
	#CONSTRUCTOR
	def __init__(self, widget, height, imgPath, themeName, init=1):
		global filledR, filledL, spacer, bottom
		
		# create the vars
		self.__widget = widget
		self.__height = height
		self.__imgPath = imgPath
		self.__themeName = themeName
		
		# create a background bar with theme name, path to images, minimum width, height, and side width
		self.__myBgBar = xBgBar(themeName+".theme", imgPath, 600, height, 26)
		# create new image object
		self.__myImgDict = objDict()
		# create new config menu object
		self.__myConfigDict = objDict()
		# kmenu obj    
		self.__myImgDict['kmenu'] = imgObj("kmenu","dcop kicker kicker popupKMenu 0",imgPath,32,28,1,2,0) 
		# finder obj & config menu
		self.__myImgDict['finder'] = imgObj("finder", "kfind", imgPath, 32, 26, 1, 4 )
		self.__myConfigDict['finder'] = configObj( "finder", "Enable finder" )
		# logoff obj & config menu
		self.__myImgDict['logoff']=imgObj("logoff", "dcop kdesktop KDesktopIface logout",imgPath,32, 30, 1,2 )
		self.__myConfigDict['logoff'] = configObj( "logoff", "Enable logoff" )
		# help obj & config menu
		self.__myImgDict['help'] = imgObj("help", "khelpcenter", imgPath, 32, 24, 1, 4 )
		self.__myConfigDict['help'] = configObj( "help", "Enable help" )
		# date config menu
		self.__myConfigDict['datetime'] = configObj( "datetime", "Enable date and time" )
		# number of system tray icons
		self.__myConfigDict['numsys'] = configObj( "numsys", "Choose number of system tray icons" )
		# font light or dark config menu
		# myConfigDict['nfontcolor'] = configObj( "nfontcolor", "Choose the font color" )
		# the date with fontSize,fontFamily,fontColor,cmd,imgPath,xpos,ypos,imgXpos,imgYpos,width,height
		self.__myDate = xDateTime(12, "Sans", 255, "kcmshell clock", imgPath, 0, 9, 0, 4, 140, height)
		# the current tasks fontSize,fontColor,imgPath,mxPos,myPos,pyPos,tyPos,txtWidth,width,height
		self.__myCurrTasks = xCurrTasks(12, "Sans", 255,imgPath,0,1,3,8,72,102,height,spacer)
		# the sys tray with num of tray icons, height, xpos, ypos
		self.__mySysTray = xSysTray(24,0,3) 
		
		# load the xBar
		self.load(init)
	
	# delete the objects
	def __del__( self ):
		try:
			self.__myBgBar = 0
			self.__myDate = 0
			self.__myCurrTask = 0 
			
			imgList = self.__myImgDict.keys()
			
			for k in imgList:
				self.__myImgDict[k] = 0
				
			self.__myImgDict = 0
			self.__myConfigDict = 0
			self.__mySysTray = 0
			
			gc.collect()
		except:
			print 'Could not destroy objects'
			
	# destroy the xBar
	def destroy(self):
		# hide objects
	        self.__myBgBar.hide(self.__widget)
		self.__mySysTray.hide( self.__widget )
		self.__myDate.visible( self.__widget, 0)
	    
	        # delete objects
		self.__myBgBar.destroy(self.__widget)	
		self.__myDate.destroy(self.__widget)
		self.__myCurrTasks.destroy( self.__widget)
		
		imgList = self.__myImgDict.keys()
			
		for k in imgList:
		        self.__myImgDict[k].visible( self.__widget, 0 ) 
		        self.__myImgDict[k].destroy( self.__widget )
			
		self.__del__()
	        
	# called to load xBar
	def load( self, init ):
		global filledR, filledL, spacer, bottom
		# route all output through the Logger
		logger = xLogger( self.__widget)
		sys.stdout = logger
		sys.stderr = logger
		
		# set width and position 
		self.__myBgBar.set(self.__widget)
		# create the bars
		self.__myBgBar.create(self.__widget)
		
		# set xpos to the width
		xpos = self.__myBgBar.getSideWidth() - spacer
		
		# prepare a reverse list of img object keys
		reverseImgList = self.__myImgDict.keys()
		reverseImgList.reverse()
		
		# prepare double spacer
		doublespacer = spacer*2
		
		# Cycle through image objects
		for k in reverseImgList:
			self.__myImgDict[k].create( self.__widget, xpos )
			xpos = xpos + doublespacer + len(self.__myImgDict[k])
			
		#  Adjust the filled space	
		filledL = xpos 	
		filledR = self.__myBgBar.getSideWidth() 
		
		#  Create the date and time
		self.__myDate.create(self.__widget, len(self.__myBgBar) - filledR) 
		
		#  Adjust the filled space
		filledR = filledR +  len(self.__myDate)
		
		#  Create the system tray
		self.__mySysTray.create(self.__widget, len(self.__myBgBar) - filledR)
		
		#  Adjust the filled space
		filledR = filledR +  len(self.__mySysTray)
		
		# Adjust Filled Space
		filledLR = filledL + filledR + spacer

		# Prepare a list of configure object keys
		configList = self.__myConfigDict.keys()
		
		# Create tasks
		self.__myCurrTasks.set( self.__widget, filledL, len(self.__myBgBar) - filledLR)
		
		# Cycle through configure objects
		for k in configList:
		    if( init == 1 ):
			self.__myConfigDict[k].create(self.__widget)
		    self.menuConfigOptions( self.__myConfigDict[k].getName(), 1)
		
		# Create tasks
		self.__myCurrTasks.create(self.__widget)
	
	# called to reload the xBAr
	def reload( self ):
	    # destroy the widget
	    self.destroy()
	    # initialize the widget
	    self.__init__(self.__widget, self.__height, self.__imgPath, self.__themeName, 0)
	    
	    # hide objects
	    self.__myBgBar.hide(self.__widget)
	    self.__mySysTray.hide( self.__widget )
		
	    # redraw self.__widget
	    karamba.redrawWidget(self.__widget)
	    
	    # show objects
	    self.__myBgBar.show(self.__widget)
	    self.__mySysTray.show( self.__widget )
	    
	# called to update xBar
	def update( self ):
		# update the tasks
		# if my dirty python code is correct, tasks should not have to be updated here
		# myCurrTasks.update(self.__widget)
		
		# update the system tray
		reloadCheck = self.__mySysTray.update(self.__widget)
		
		if( reloadCheck == 1):
		    self.reload()
		
		#  Get date and time if not hidden
		self.__myDate.update(self.__widget)
		
	###############################################################
	#  This is called to configure the menu Options
	############################################################### 
	def menuConfigOptions( self, key, init ):
		global filledR, filledL, spacer, bottom
		# read menu configuration 
		value = karamba.readMenuConfigOption(self.__widget, key)
		
		# set xpos to the width
		xpos = self.__myBgBar.getSideWidth() - spacer
		
		# change the font color
		if key == "nfontcolor" and init <> 1:
			fontColorCmd="kdialog --title 'xBar configuration' --combobox 'Choose font color' black white red purple blue green orange"
			fontColor = commands.getoutput( fontColorCmd )
			# for k in myImgDict.keys():
			#    myImgDict[k].color(self.__widget,value)
			self.__myDate.color(self.__widget,fontColor)
			self.__myCurrTasks.color(self.__widget,fontColor)
		
		# enable or disable the date and time
		elif key == "datetime":
			self.__myDate.visible( self.__widget, value )
			# move the SysTray to original position
			filledR = self.__myBgBar.getSideWidth()+len(self.__myDate)+len(self.__mySysTray)
			self.__mySysTray.move(self.__widget,len(self.__myBgBar) - filledR)
			
		# choose number of system tray icons	
		elif key == "numsys" and init <> 1:	
			sysTrayCmd="kdialog --title 'xBar configuration' --combobox 'Choose number of system tray icons' 1 2 3 4 5 6 7 8"
			self.__mySysTray.save( commands.getoutput( sysTrayCmd ))
			self.__mySysTray.update( self.__widget )
			
		# enable or disable 
		elif self.__myImgDict.has_key( key ):
			self.__myImgDict[key].visible( self.__widget, value )
			
		# prepare a reverse list of img object keys
		reverseImgList = self.__myImgDict.keys()
		reverseImgList.reverse()
		
		# Cycle through image objects
		for k in reverseImgList:
			self.__myImgDict[k].move( self.__widget, xpos )
			xpos = xpos + len(self.__myImgDict[k])	
			
		filledL = xpos
		filledLR = filledR + filledL + spacer
		
		if( init == 1 ):
			# set tasks variables
			self.__myCurrTasks.set( self.__widget, filledL, len(self.__myBgBar) - filledLR )
		else:
			# recreate tasks
			self.__myCurrTasks.reset( self.__widget, filledL, len(self.__myBgBar) - filledLR )
		
		karamba.writeConfigEntry(self.__widget, key, str(value))
		
		# redraw self.__widget
		karamba.redrawWidget(self.__widget)
		
	# called whenever a new task has been started
	def taskAdded(self):
		# add the tasks
		self.__myCurrTasks.add(self.__widget)
	
	# called whenever a task is removed
	def taskRemoved(self):
		# remove the tasks
		self.__myCurrTasks.update(self.__widget)
	
	# called whenever a new task has gained focus.
	# task = the active task
	def activeTaskChanged(self):    
		# activate the tasks
		self.__myCurrTasks.activate(self.__widget)
	
	# called when bar is clicked
	def barClicked(self, x, y, button):
		global filledR, filledL, spacer, bottom
		self.__myCurrTasks.clicked( self.__widget, x, filledL )
		
	#This gets called when an item is clicked in a popup menu you have created.
	#  menu = a reference to the menu
	#  id = the number of the item that was clicked.
	def menuItemClicked(self, menu, id):
		self.__myCurrTasks.menuItemClicked(self.__widget, menu, id)
