###############################################################
#  Copyright by Charles Wienken
#  Licensed Under GPL
###############################################################

###############################################################
#  This import statement allows access to the karamba functions
###############################################################
import karamba
import commands
import string
import time
import gc

class xCurrTasks:
    # initialize current tasks
    def __init__(self,FSize,FFamily,FColor,imgPath,mxPos,myPos,pyPos,tyPos,txtWidth,w,h,spacing):
	self.__fontSize = FSize # size of the font
	self.__fontFamily = FFamily # font family
	self.__fontC1 = FColor # font color
	self.__fontC2 = FColor # font color
	self.__fontC3 = FColor # font color
	self.__imgPath = imgPath+"task_new.png" # image path
	self.__menu = 0 # menu is used for more than one item
	self.__menuLookup = {} # menu look up
	self.__menuXpos = mxPos # x position of menu
	self.__menuYpos = myPos # y position of menu
	self.__width = w # width of each task
	self.__height = h # height of each task
	self.__panels = [] # task images 
	self.__panelYpos = pyPos # panel y position
	self.__text = [] # task text
	self.__textXpos = [] # each tasks x position
	self.__textYpos = tyPos # task y position
	self.__textWidth = txtWidth # width of text to show
	self.__maxNumOfTasks = 0 # max of tasks
	self.__numOfTasks = 0 # num of tasks
	self.__iconID = [] # ID of the icon
	self.__removeTask = 0 # remove task
	self.__activeTask = 0 # active task
	self.__spacing = spacing # task spacer
	
    def destroy(self,widget):
	try:
	# clear all images, and icons
	    for i in range(self.__numOfTasks):
		karamba.hideImage(widget, self.__panels[i])
		karamba.hideImage(widget, self.__iconID[i])
		karamba.deleteText(widget, self.__text[i])
	    self.__numOfTasks = 0
	    self.__text = []
	    self.__iconID = []
	    self.__panels = []
	    # collect the trash and dispose of it proper like
	    gc.collect() 
	except:
	   print 'Could not cleanup task objects'
    
    # set current tasks vars
    def set(self,widget,startX,overSpace):
	self.__maxNumOfTasks = overSpace / self.__width
	self.__startXpos = startX
	    
    # reset current tasks
    def reset(self,widget,startX,overSpace):
	self.__maxNumOfTasks = overSpace / self.__width
	self.__startXpos = startX

	# clear all tasks, images, and icons
	self.destroy(widget)

	# create list of tasks
	self.create(widget)

    # update to show curr tasks
    def add(self,widget):
	#  get the list of tasks
        taskList = karamba.getTaskList(widget)
	
        # get the length of all tasks
	length = len(taskList)
	
	# check the length
	if (self.__maxNumOfTasks < length):
            length = self.__maxNumOfTasks
	
	knownGroups = {}
        groups = []
	groupRefs = {}
        activeGroup = -1
        groupCount = 0
	
	# build groups list that contains sub-lists of taskInfo sorted by group
        for j in range(len(taskList)):
            taskinfo = karamba.getTaskInfo(widget, taskList[j])
	    if knownGroups.has_key(taskinfo[2])	 == 0 and groupCount < length:
		knownGroups[taskinfo[2]] = 1
		thisGroup = []
		thisGroup.append(taskinfo)
		groupRefs[taskinfo[2]] = index = len(groups)
		groups.append(thisGroup)
		if( groupCount >= self.__numOfTasks):
		  x = self.__startXpos + self.__width*groupCount
		  self.__textXpos.insert(groupCount,x)
                  self.__panels.insert(groupCount,karamba.createImage(widget,x,self.__panelYpos, self.__imgPath))
		  text = groups[groupCount][0][0]
                  self.__text.insert(groupCount,karamba.createText(widget,x+25,self.__textYpos,self.__textWidth,self.__height,text))
                  self.__iconID.insert(groupCount,karamba.createTaskIcon(widget,x+self.__spacing,self.__textYpos,taskList[j]))
	          karamba.resizeImageSmooth(widget, self.__iconID[groupCount], 15, 15)
	          karamba.changeTextSize( widget,self.__text[groupCount],self.__fontSize )
	          karamba.changeTextFont(widget, self.__text[groupCount], self.__fontFamily)
		karamba.changeTextShadow(widget, self.__text[groupCount], 0)
                karamba.changeTextColor(widget,self.__text[groupCount],self.__fontC1,self.__fontC2,self.__fontC3)
		if (taskinfo[7] == 1):
		    activeGroup = groupCount		  
		groupCount = groupCount + 1
	    elif groupCount < length:
		knownGroups[taskinfo[2]] = knownGroups[taskinfo[2]] + 1
		thisGroup = groups[groupRefs[taskinfo[2]]]
		thisGroup.append(taskinfo)
		if (taskinfo[7] == 1):
		    activeGroup = groupRefs[taskinfo[2]]	    
		    
	self.__numOfTasks = groupCount
	
	# make active group look different
	if (activeGroup <> -1):
	    karamba.changeTextShadow(widget, self.__text[activeGroup], 1)
	    karamba.changeTextColor(widget, self.__text[activeGroup], 239, 220, 11)
	    karamba.rotateImage(widget, self.__panels[activeGroup], 180)
	
	# redraw widget
	karamba.redrawWidget(widget)
	
    # create the tasks
    def create(self,widget):
	#  get the list of tasks
        taskList = karamba.getTaskList(widget)
	
        # get the length of all tasks
	length = len(taskList)
	
	# check the length
	if (self.__maxNumOfTasks < length):
            length = self.__maxNumOfTasks
	
	knownGroups = {}
        groups = []
	groupRefs = {}
        activeGroup = -1
        groupCount = 0
	
	# build groups list that contains sub-lists of taskInfo sorted by group
        for j in range(len(taskList)):
            taskinfo = karamba.getTaskInfo(widget, taskList[j])
	    if knownGroups.has_key(taskinfo[2])	 == 0 and groupCount < length:
		knownGroups[taskinfo[2]] = 1
		thisGroup = []
		thisGroup.append(taskinfo)
		groupRefs[taskinfo[2]] = index = len(groups)
		groups.append(thisGroup)
		x = self.__startXpos + self.__width*groupCount
		self.__textXpos.insert(groupCount,x)
                self.__panels.insert(groupCount,karamba.createImage(widget,x,self.__panelYpos, self.__imgPath))
		text = groups[groupCount][0][0]
                self.__text.insert(groupCount,karamba.createText(widget,x+25,self.__textYpos,self.__textWidth,self.__height,text))
		karamba.changeTextShadow(widget, self.__text[groupCount], 0)
                karamba.changeTextColor(widget,self.__text[groupCount],self.__fontC1,self.__fontC2,self.__fontC3)
	        karamba.changeTextSize( widget,self.__text[groupCount],self.__fontSize )
	        karamba.changeTextFont(widget, self.__text[groupCount], self.__fontFamily)
                self.__iconID.insert(groupCount,karamba.createTaskIcon(widget,x+self.__spacing,self.__textYpos,taskList[j]))
	        karamba.resizeImageSmooth(widget, self.__iconID[groupCount], 15, 15)
		if (taskinfo[7] == 1):
		    activeGroup = groupCount		  
		groupCount = groupCount + 1
	    elif groupCount < length:
		knownGroups[taskinfo[2]] = knownGroups[taskinfo[2]] + 1
		thisGroup = groups[groupRefs[taskinfo[2]]]
		thisGroup.append(taskinfo)
		if (taskinfo[7] == 1):
		    activeGroup = groupRefs[taskinfo[2]]	    
		    
	self.__numOfTasks = groupCount
	
	# make active group look different
	if (activeGroup <> -1):
	    karamba.changeTextShadow(widget, self.__text[activeGroup], 1)
	    karamba.changeTextColor(widget, self.__text[activeGroup], 239, 220, 11)
	    karamba.rotateImage(widget, self.__panels[activeGroup], 180)
	
	# redraw widget
	karamba.redrawWidget(widget)
    
    # update to show curr tasks
    def update(self,widget):
        # clear all tasks, images, and icons
	self.destroy(widget)

	# create list of tasks
	self.create(widget)
	
    # activate to show curr tasks
    def activate(self,widget):	
	#  get the list of tasks
        taskList = karamba.getTaskList(widget)
	
        # get the length of all tasks
	length = len(taskList)
	
	# check the length
	if (self.__maxNumOfTasks < length):
            length = self.__maxNumOfTasks
	
	knownGroups = {}
        groups = []
	groupRefs = {}
        activeGroup = -1
        groupCount = 0
	
	# build groups list that contains sub-lists of taskInfo sorted by group
        for j in range(len(taskList)):
            taskinfo = karamba.getTaskInfo(widget, taskList[j])
	    if knownGroups.has_key(taskinfo[2])	 == 0 and groupCount < length:
		knownGroups[taskinfo[2]] = 1
		thisGroup = []
		thisGroup.append(taskinfo)
		groupRefs[taskinfo[2]] = index = len(groups)
		groups.append(thisGroup)
		karamba.changeTextShadow(widget, self.__text[groupCount], 0)
                karamba.changeTextColor(widget,self.__text[groupCount],self.__fontC1,self.__fontC2,self.__fontC3)
	        karamba.removeImageTransformations(widget, self.__panels[groupCount])
		if (taskinfo[7] == 1):
		    activeGroup = groupCount		  
		groupCount = groupCount + 1
	    elif groupCount < length:
		knownGroups[taskinfo[2]] = knownGroups[taskinfo[2]] + 1
		thisGroup = groups[groupRefs[taskinfo[2]]]
		thisGroup.append(taskinfo)
		if (taskinfo[7] == 1):
		    activeGroup = groupRefs[taskinfo[2]]	    
	
	# make active group look different
	if (activeGroup <> -1):
	    karamba.changeTextShadow(widget, self.__text[activeGroup], 1)
	    karamba.changeTextColor(widget, self.__text[activeGroup], 239, 220, 11)
	    karamba.rotateImage(widget, self.__panels[activeGroup], 180)
	    
	# redraw widget
	karamba.redrawWidget(widget)
	
    # called when widget is clicked
    def clicked( self, widget, x, filledL ):
	taskSelected = (x-filledL)/self.__width

        # Make sure its a valid task
        if (0 <= taskSelected < self.__maxNumOfTasks):
	    
	    #free last menu
	    karamba.deleteMenu(widget, self.__menu)

	    #create new menu
	    self.__menu = karamba.createMenu(widget)
	    self.__menuLookup = {}
	    
	    #  Update the list of tasks
	    taskList = karamba.getTaskList(widget)
	
	    # get the length of all tasks
	    length = len(taskList)
	
	    if (self.__maxNumOfTasks < length):
		length = self.__maxNumOfTasks


	    knownGroups = {}
	    groups = []
	    groupRefs = {}
	
	    for j in range(len(taskList)):
		taskinfo = karamba.getTaskInfo(widget, taskList[j])
	
		if knownGroups.has_key(taskinfo[2]) == 0:
		    knownGroups[taskinfo[2]] = 1
		    thisGroup = []
		    thisGroup.append(taskinfo)
		    groupRefs[taskinfo[2]] = len(groups)
		    groups.append(thisGroup)
		else:
		    knownGroups[taskinfo[2]] = knownGroups[taskinfo[2]] + 1
		    thisGroup = groups[groupRefs[taskinfo[2]]]
		    thisGroup.append(taskinfo)

	    if taskSelected < len(groups):
		taskGroup = groups[taskSelected]
	    

		if len(taskGroup) == 1:
		    #only one task in group - just switch to that app
		    ## perform the task action (see Task API for list of numbers)
		    karamba.performTaskAction(widget, taskGroup[0][8], 8)
		    
		else:		    		
		    #more than one task in this group, make a popup
		    for task in taskGroup:
			#if it's minimized, put []'s around name
			if task[5] == 1:
			    item = karamba.addMenuItem(widget,self.__menu,"[ "+task[0]+" ]",task[2]) 
			else:
			    item = karamba.addMenuItem(widget,self.__menu, task[0], task[2]) 
			    #save the taskInfo item for later use
			    #so we will know info for the task that
			    #will be clicked in the callback
			self.__menuLookup[item] = task

		    numOfItems = len(taskGroup)
		    self.__menuXpos = self.__textXpos[taskSelected]
		    # self.__menuXpos = filledL+(self.__spacing*taskSelected)
		    karamba.popupMenu(widget, self.__menu, self.__menuXpos, self.__menuYpos*32 )

	if (taskSelected == self.__maxNumOfTasks):
	    karamba.toggleShowDesktop(widget)
	
    # menu item was clicked
    def menuItemClicked(self,widget, menu, id):
        taskinfo = self.__menuLookup[id]
	karamba.performTaskAction(widget, taskinfo[8], 8)
	
    # change the font color
    def color(self,widget,color):
	c1 = 255
	c2 = 255
	c3 = 255
	    
	if( color == "black" ):
	    c1 = 0
	    c2 = 0
	    c3 = 0
	elif( color== "red" ):
	    c1 = 255
	    c2 = 0
	    c3 = 0
	elif( color == "purple" ):
	    c1 = 117
	    c2 = 0
	    c3 = 146
	elif( color == "blue" ):
	    c1 = 11
	    c2 = 11
	    c3 = 193
	elif( color == "green" ):
	    c1 = 10
	    c2 = 145
	    c3 = 64
	elif( color == "orange" ):
	    c1 = 255
	    c2 = 168
	    c3 = 0
	    
        self.__fontC1 = c1 # font color 1
	self.__fontC2 = c2 # font color 2
	self.__fontC3 = c3 # font color 3
	self.activate(widget,0)	
	
