###############################################################
#  Copyright by Charles Wienken
#  Licensed Under GPL
###############################################################

###############################################################
#  This import statement allows access to the karamba functions
###############################################################
import karamba
import commands
import string
import time
import sys 

# try to import KWinModule from kdecore -- this requires PyQt and PyKDE
try:
    from kdecore import KWinModule
    pyKDERunning = 1
except:
    pyKDERunning = 0

###############################################################
#  xSysTray Class
###############################################################		

class xSysTray:
    # initialize the sys tray
    def __init__(self, height, xpos, ypos):
	global pyKDERunning
	self.__autoResize = pyKDERunning
	self.__width = 0 # width of sys tray
	self.__height = height # sys tray height
	self.__xPos = xpos # sys tray x position
	self.__yPos = ypos # sys tray y position
	self.__filehandler = 0 # file handler for system tray
	self.__filename = "SysTrayIcons" # file name for system tray
	self.__filepath = "" # file path for system tray
	self.__iconSize = height # size of tray icons
	
    # get the width of the system tray
    def __len__(self):
	return self.__width
    
    # get the number of icons of the system tray
    def getNumOfIcons(self):
	return self.__width / self.__iconSize
	    
    # function to create the system tray       
    def create(self,widget,x):
	self.__filepath = karamba.getThemePath(widget)		
	try:
	    if self.__autoResize:
		#use auto-resize
		self.kWinModule = KWinModule()
		self.__width = len(self.kWinModule.systemTrayWindows()) * self.__iconSize
	    else:
		#do not use auto-resize, use manually-entered number, iconSize pixels per icon
	        print "WARNING: Could not load PyKDE. The SysTrayPanel will not be able to auto-resize without it. However, it will still run."
		# open file for reading
		self.__filehandler = file( self.__filepath + self.__filename, "r")
		#read width from file
		STwidth = self.__filehandler.read()
		# close the file	
		self.__filehandler.close()
		self.__width = int(STwidth) * self.__iconSize
    	    # create System Tray
	    self.__xPos = x + self.__width
    	    karamba.createSystray(widget, self.__xPos, self.__yPos,self.__width,self.__height)
        except:
	   print 'Systray does not work with this version of superkaramba'	        
    
    # function to update the system tray
    def update(self,widget): 
	reload = 0
	try:
	    if self.__autoResize:
		#use auto-resize
		self.kWinModule = KWinModule()
		newWidth = len(self.kWinModule.systemTrayWindows()) * self.__iconSize
	    else:
		# open file for reading
		self.__filehandler = file(self.__filepath + self.__filename, "r")
		#read width from file
		STwidth = self.__filehandler.read()
		# close the file	
		self.__filehandler.close()
		newWidth = int(STwidth) * self.__iconSize
	    if newWidth != self.__width:
		self.__xPos = self.__xPos - newWidth + self.__width
		self.__width = newWidth
		# hide the system tray
		# karamba.hideSystray(widget)
		# move Sys Tray
		# karamba.moveSystray(widget, self.__xPos, self.__yPos,newWidth,self.__height)
		# show the system tray
		# karamba.showSystray(widget)
		# redraw widget
		# karamba.redrawWidget(widget)
		reload = 1
	except:
	   print 'Systray does not work with this version of superkaramba'
	return reload
	    
    # function to save size of system tray
    def save(self, n):
	try:
	    # open file for writing
	    self.__filehandler = file(self.__filepath + self.__filename, "w")
	    #write width to file
	    self.__filehandler.write(n)
	    # close the file
	    self.__filehandler.close()
	except:
	   print 'Systray does not work with this version of superkaramba'
	    
    # function to hide the system tray	   
    def hide(self,widget):
	# hide the system tray
	karamba.hideSystray(widget)
	
    # function the show the system tray
    def show(self,widget):
	# show the system tray
	karamba.showSystray(widget)
	
    # function to save the x pos and move the system tray
    def move(self, widget, x):
	try:
	    # hide the system tray
	    karamba.hideSystray(widget)
	    self.__xPos = x
	    karamba.moveSystray(widget, self.__xPos, self.__yPos,self.__width,self.__height)
	    # show the system tray
	    karamba.showSystray(widget)
	    # redraw widget
	    karamba.redrawWidget(widget)
	except:
	   print 'Systray does not work with this version of superkaramba'
	
